#!/tvbin/tivosh
# $Id: httpd-tt.tcl,v 1.6 2005/08/17 02:16:40 davidlallen Exp $
# added loading support for options to cull based on pay per view, not received 
#   and xm channel types.

namespace eval TWP {
  variable NAME	"TivoWebPlus"
  variable VERSION "1.3.1"
  variable DIST_VERSION ""
  variable OPTIONS_MENU ""
  variable MAIN_MENU ""
  variable STYLE ""
}

# sources a file and strips comments and indents... saves LOTS of space
#9728 3260
proc mySource {f} {
  if {[file size $f] < 50000} {
    set fd [open $f r]
    set s [read $fd]
    close $fd
    regsub -all "\n\[ \t\]*(#\[^\n\]*)?" $s "\n" s
    set $::errorCode ""
    set $::errorInfo ""
    if {[catch { uplevel 1 "$s" }]==1} {
      puts "$::errorCode $::errorInfo"
    }
  } else {
    uplevel 1 "source $f"
  }
}

#setpri fifo 1
catch {EnableTransactionHoldoff true}

set mfsVerboseG 0

mySource $tcl_library/tv/log.tcl
mySource $tcl_library/tv/mfslib.tcl
mySource $tcl_library/tv/dumpobj.tcl

global source_dir
set source_dir [file dirname [info script]]

if {[file exists $source_dir/.dist/version]} {
  set fd [open $source_dir/.dist/version "r"]
  gets $fd TWP::DIST_VERSION
  close $fd
}

mySource $source_dir/libs/util.itcl

# disable keyname echoing to controling tty
set verboseSendKeyG 0

puts "$TWP::NAME - v$TWP::VERSION ($TWP::DIST_VERSION)\nThe program comes with ABSOLUTELY NO WARRANTY.\nThis program is licensed under the GPL.\nSee the 'docs/copyright' and 'docs/index.html' files\nfor copyright and credit information."
catch {puts "PoolSize=$env(TIVOSH_POOLSIZE)"}

proc register_module {url sdesc ldesc} {
  global module_list
  if { [string first "." $url] == -1 } {
    lappend module_list [list "$url/" $sdesc $ldesc]
  } else {
    lappend module_list [list $url $sdesc $ldesc]
  }
}

proc make_menu {} {
  global module_list

  # set module_list [linsert $module_list 0 [list "" "Main Menu" ""]]
  # lappend $module_list 0 [list "quit" "Quit" "Terminate $TWP::NAME"]

  set TWP::OPTIONS_MENU "\n"
  #append TWP::OPTIONS_MENU [html_link "/" "Main Menu"]
  #append TWP::OPTIONS_MENU "\n"

  set TWP::MAIN_MENU "
<TABLE class=\"main_menu\" border=0 cellpadding=0 cellspacing=0>
<TR ALIGN=CENTER><TH COLSPAN=2>$TWP::NAME v$TWP::VERSION</TH></TR>"
  append TWP::MAIN_MENU "\n"

  foreach module $module_list {
    append TWP::OPTIONS_MENU [html_link "/[lindex $module 0]" [lindex $module 1]]
    append TWP::OPTIONS_MENU "\n"

    append TWP::MAIN_MENU [tr "ALIGN=LEFT" [td [html_link "/[lindex $module 0]" [lindex $module 1]]] [td [lindex $module 2]]]
    append TWP::MAIN_MENU "\n"
  }

  append TWP::OPTIONS_MENU [html_link "/restart" "Restart"]

  append TWP::MAIN_MENU [tr "ALIGN=CENTER" [td "COLSPAN=2" ""]]
  append TWP::MAIN_MENU "\n"
  append TWP::MAIN_MENU [tr "ALIGN=LEFT" [td [html_link "/restart" "Restart"]] [td "Reload or Quit $TWP::NAME"]]
  append TWP::MAIN_MENU "\n"
  append TWP::MAIN_MENU [html_table_end]

}

proc action_restart {chan path env} {
  global reload

  if {[string index $path 0] == "/"} {
    set path [string range $path 1 end]
  }

  if { $path == "" } {
    puts $chan [html_start "Restart"]

    puts $chan [html_table_start "" "" "ALIGN=TOP"]
    puts $chan [tr "" [th "" "Restart"]]
    puts $chan [tr "" [td [html_link "/restart/1" "Quick Reload"]]]
    puts $chan [tr "" [td [html_link "/restart/0" "Full Reload"]]]
    puts $chan [tr "" [td [html_link "/quit" "Quit"]]]
    puts -nonewline $chan [html_table_end]
    puts $chan [html_end]
  } else {
    set reload $path
    # puts "Loading modules..."
    puts $chan [html_start "Reload"]
    puts $chan "<B>Reload Complete</B>"
    puts $chan [html_end]
  }
}

proc action_quit {chan dummy env} {
  set ::reload "quit"
  puts $chan [html_start ""]
  puts $chan "Server has terminated."
  puts $chan [html_end]
}


proc action_ {chan dummy env} {
  puts $chan [html_start ""]
  puts $chan "$TWP::MAIN_MENU"
  puts $chan "<P>"
  puts $chan [html_link "http://www.tivoland.com/forum" "<i>Alternative EPG Support Forum</i>" "target=_blank"]
  puts $chan [html_end]
}

proc register_content_handler {suffix mimetype directory binary function} {
  global content_suffix_list
  global content_handler_list

  if {[lsearch $content_suffix_list $suffix] == -1} {
    lappend content_suffix_list $suffix
    lappend content_handler_list [list $mimetype $directory $binary $function]
  }
}

proc serve_file {chan filename head_req last_modified} {
  global db
  global source_dir
  global content_suffix_list
  global content_handler_list
  global tivoswversion
  global startuptime

  set suffix [file extension $filename]
  if {[string index $suffix 0] == "."} {
    set suffix [string range $suffix 1 end]
  }
  set index [lsearch $content_suffix_list $suffix]
  set clist [lindex $content_handler_list $index]
  set mimetype [lindex $clist 0]
  set directory [lindex $clist 1]
  set binary [lindex $clist 2]

  set fd ""
  catch { set fd [open "$directory/$filename" "r"] }
  if { $fd != "" } {
    set moddate [file mtime "$directory/$filename"]
    if { $head_req == 1 } {
      print_html_header_200 $chan $mimetype $moddate
    } elseif { $last_modified == $moddate } {
      print_html_header_304 $chan
    } else {
      print_html_header_200 $chan $mimetype $moddate
      if { $binary == 1 } {
        fconfigure $chan -translation binary
      }
      if { $fd != "" } {
        if { $binary == 1 } {
          fconfigure $fd -translation binary
        }
        fcopy $fd $chan
      }
    }
    if { $fd != "" } {
      close $fd
    }
  } else {
    print_html_header_404 $chan
  }
}

proc serve_image {chan imagename head_req last_modified} {
  global db
  global source_dir
  global content_suffix_list
  global content_handler_list

  set suffix [file extension $imagename]
  if {[string index $suffix 0] == "."} {
    set suffix [string range $suffix 1 end]
  }
  set index [lsearch $content_suffix_list $suffix]
  set clist [lindex $content_handler_list $index]
  set mimetype [lindex $clist 0]
  set directory [lindex $clist 1]

  set fd ""
  set imagedata ""
  set moddate ""
  catch { set fd [open "$directory/$imagename" "r"] }
  if { $fd != "" } {
    set moddate [file mtime "$directory/$imagename"]
  } else {
    # Check MFS resources
    set imagename [file rootname $imagename]
    catch {
      RetryTransaction {
        set imagefsid [lindex [mfs find "/Resource/Image/$imagename"] 0]
        set moddate [mfs moddate $imagefsid]
        if {$moddate != $last_modified} {
          set obj [db $db openid $imagefsid]
          if {$obj != ""} {
            set imageid [dbobj $obj get File]
            if {$imageid != ""} {
              set imagedata [mfs get $imageid]
            }
          }
        } else {
          set imagedata "cached"
        }
      }
    }
  }
  if { ($fd != "") || ($imagedata != "") } {
    if { $head_req == 1 } {
      print_html_header_200 $chan $mimetype $moddate
    } elseif { $last_modified == $moddate } {
      print_html_header_304 $chan
    } else {
      print_html_header_200 $chan $mimetype $moddate
      fconfigure $chan -translation binary
      if { $fd != "" } {
        fconfigure $fd -translation binary
        fcopy $fd $chan
      } else {
        puts -nonewline $chan $imagedata
      }
    }
    if { $fd != "" } {
      close $fd
    }
  } else {
    print_html_header_404 $chan
  }
}

proc decode_upload {chan content_boundary content_length} {
  global source_dir
  upvar post_data post_data
  global block

  set boundary "--$content_boundary"
  set blen [expr [string length $boundary]+2] 

  set len $content_length
  set block 2048

  set fp "" 
  set key ""
  set header ""

  fconfigure $chan -translation binary
  set buffer ""

  while { $len > 0 } {
    set rlen $block
    if {$len < $rlen} { set rlen $len }
    incr len -$rlen

    append buffer [read $chan $rlen] 
    set ret [string first $boundary $buffer]

    while { $ret != -1 } {
      if {$fp != ""} {
        if {$ret > 0} {puts -nonewline $fp "[string range $buffer 0 [expr $ret -1]]"}
        close $fp
        set buffer [string range $buffer $ret end]
        set ret 0
      } elseif {$key != ""} {
        if {$post_data != ""} { append post_data "\&" }
        append post_data "$key=[string range $buffer 0 [expr $ret -1]]"
        set buffer [string range $buffer $ret end]
        set ret 0
      }
      set fp ""
      set key ""

      # Crop off the boundary string
      # Careful not to bisect headers
      # ie "--BOUND Con<EOF>"
      set tmp [expr $ret+$blen]
      set ret [string first "\r\n\r\n" $buffer]
      if {$ret > $tmp || $ret < 0} {
        set buffer [string range $buffer $tmp end]
        incr ret -$tmp
        set header ""
      }
      
      if {$ret > 0} { ;# Parse header
        if { $header == "" } {
          set filename ""
          set name ""
          set key ""
          set header "[string range $buffer 0 [expr $ret -1]]"

          if {[regexp -nocase {^Content-Disposition:.*\ name="([^"]*)\"} $header junk name] ==1} {
            if {[regexp -nocase {.*filename="([^"]*)\"} $header junk filename] == 1} {
              #some browsers give the full path
              #discard path and give just the filename.
              regexp -nocase {[\\/]?([^\\/]*)$} $filename dummy filename
            }
            set key [httpMapReply $name]
            if {$filename != ""} {
              set fp [open "$source_dir/uploads/$filename" w]
              fconfigure $fp -translation binary
              if {$post_data != ""} { append post_data "\&" }
              append post_data "$key=$filename"
            }
          } else {
            #clear invalid headers
            set header ""
          }
        }
        set buffer [string range $buffer [expr $ret+4] end]
      }

      if {$boundary == "--$content_boundary"} {
        set boundary "\r\n--$content_boundary"
        set blen [expr [string length $boundary]+2]
      }

      #setup the whole loop again
      set ret [string first $boundary $buffer]
    }
    set slen [string length $buffer]
    # spit out block bytes if possible, otherwise just the remaining bytes
    # we need to preserve $blen bytes in the buffer to deal with edge
    # conditions
    # NOTE: if we aren't writing to a file we just let the buffer fill 
    if { $fp != "" } {
      if {$slen > $block + $blen || $rlen < $block } {
        puts -nonewline $fp "[string range $buffer 0 [expr $block -1]]"
        set buffer [string range $buffer $block end]
      }
    }
  }
  fconfigure $chan -buffering none -blocking 1
}

proc session {chan addr port} {
  global HTTP_HOST
  global REMOTE_ADDR
  global IE_alpha_trans_fix
  set auth 0
  set head_req 0
  set post_req 0
  set post_data ""
  set if_modified_since ""
  set last_modified 0
  set content_length 0
  set HTTP_USER_AGENT ""
  set HTTP_HOST ""
  set REMOTE_ADDR $addr
  set IE_alpha_trans_fix 0

  # added by JAKE -
  # http_upload - flag indicates if body has multipart form to parse
  # content_boundary - holds the boundary text between the parts 
  set http_upload 0
  set content_boundary ""

  fconfigure $chan -buffering none -blocking 1
  while {[gets $chan line] >= 0} {
    if {$line == ""} {
      if { $post_req == 1 } {
        #added by JAKE - check to see if there is a multipart form
        #if so, parse it, if not, handle as TivoWeb always did.
        if {$http_upload == 1} {
          decode_upload $chan $content_boundary $content_length
        } else {
          #Just another POST (standard httpd-tt code)        
          append post_data [read $chan $content_length]
        }
      }
      break
    }

    # DEBUG
    #set fd [open /tmp/headers a]
    #puts $fd "$line"
    #close $fd

    #if {[regexp -nocase {^kill} $line]  == 1} {
    #    set quit "puts killed"
    #    return
    #}
    if {[regexp -nocase {^get +([^\?]*)\?*(.*) +http/[0-9]+\.[0-9]+.?$} $line dummy path post_data] == 1} {
      continue
    }
    if {[regexp -nocase {^head +([^\?]*)\?*(.*) +http/[0-9]+\.[0-9]+.?$} $line dummy path post_data] == 1} {
      #    set quit 1
      #    return
      set head_req 1
      continue
    }
    if {[regexp -nocase {^post +([^\?]*)\?*(.*) +http/[0-9]+\.[0-9]+.?$} $line dummy path post_data] == 1} {
      set post_req 1
      continue
    }
    if {[regexp -nocase {^Content-length: ([0-9]+)$} $line dummy content_length] == 1} {
      continue
    }

    if {[regexp -nocase {^Host: (.*)$} $line dummy HTTP_HOST] == 1} {
      continue
    }

    # Currently not required
    #if {[regexp -nocase {^Referer: (.*)$} $line dummy HTTP_REFERER] == 1} {
    #  continue
    #}

    # User-Agent added by John1980 as this will allow us to
    # fix PNG alpha transparency support in IE 5.5 and 6.
    if {[regexp -nocase {^User-Agent: (.*)$} $line dummy HTTP_USER_AGENT] == 1} {
      if {[regexp {.*MSIE [5-6].*} $HTTP_USER_AGENT]} {
        set IE_alpha_trans_fix 1
      }
      continue
    }

    #Content-type added by JAKE - The content-type header is present
    #and has multipart/form data.  set the http_upload flag, and
    #get the boundary string.
    if {[regexp -nocase {^Content-Type: multipart/form-data; boundary=(.*)$} $line junk content_boundary] == 1} {
      set http_upload 1
      continue
    }

    if {$::userpass != ""} {
      if {[regexp -nocase {^Authorization: +Basic +([A-Za-z0-9+/=]+)$} $line dummy authcode] == 1} {
        set authdecode [base64dec $authcode]
        if { $authdecode == $::userpass } {
          set auth 1
        }
        continue
      }
    }
    if {[regexp -nocase {^If-Modified-Since: ([^;]*).*$} $line dummy if_modified_since] == 1} {
      set last_modified [clock scan $if_modified_since]
      continue
    }
  }

  if {$::hosts_allow != ""} {
    set found 0
    foreach Net "$::hosts_allow 127.0.0.1" {
      if {[string first $Net $REMOTE_ADDR] == 0} {
        set found 1
        break
      }
    }
    if {$found != 1} {
      print_html_header_403 $chan
      catch {flush $chan}
      catch {close $chan}
      return
    }
  }

  if {$::userpass != "" && $auth == 0} {
    if {$::lan_auth == 0} {
      regsub {\.[0-9]+$} $::ip_address {} Net1
      regsub {\.[0-9]+$} $REMOTE_ADDR  {} Net2
      if {$Net1 == $Net2} {set auth 1}
      unset Net1 Net2
    }

    if {$::rss_auth == 0} {
      if {[regexp {\.rss|rss-style.xsl} $path] == 1} {
        set auth 1
      }
    }

    if {$auth == 0} {
      print_html_header_401 $chan
      catch {flush $chan}
      catch {close $chan}
      puts "$REMOTE_ADDR - - \[[clock format [clock seconds] -format "%d/%b/%Y:%T" -gmt true]\] \"Authentication Failed\""
      return
    }
  }

  set path [url_decode $path]
  puts "$REMOTE_ADDR - - \[[clock format [clock seconds] -format "%d/%b/%Y:%T" -gmt true]\] \"$path\""
  if {[regexp -nocase {/([^/\\]+)\.([A-Z0-9]+)$} $path dummy filename suffix] == 1} {
    set index [lsearch $::content_suffix_list $suffix]
    if { $index >= 0 } {
      set function [lindex [lindex $::content_handler_list $index] 3]
      if {[catch [$function $chan "$filename.$suffix" $head_req $last_modified] error]} {
        print_html_error $chan "$function '$filename.$suffix' '$head_req' '$last_modified'" $error
        puts $chan [html_end]
      }
    } else {
      print_html_header_404 $chan
    }
  } else {
    regsub -all {\\.} $path {.} path
    if {$head_req == 1} {
      catch {close $chan}
      return
    }

    if {$::url_prefix != ""} {
      if {[regexp "^/$::url_prefix" $path] == 1} {
        set path [string range $path [string length $::url_prefix] end]
      }
    }

    if {[regexp {^/([-_A-Za-z0-9]*)(.*)} $path dummy action part] == 1 &&
        [info procs "action_$action"] == "action_$action"} {
      #
      # Cleanup evrc if not required
      #
      if {[regexp {screen|sendkey} $action] != 1} {catch {unset ::evrc}}

      print_html_header_200 $chan "text/html; charset=utf-8" ""
      set env [parse_post $post_data]
      if {[catch {eval {::action_$action $chan $part $env}}]} {
        print_html_error $chan "action_$action '$part' '$env'" $::errorInfo
        puts $chan [html_end]
      }
    } else {
      print_html_header_404 $chan
    }
  }
  catch {flush $chan}
  catch {close $chan}
}

proc binary_compare {value default} {
  set value [string tolower $value]
  if {[string compare "yes" $value] == 0 ||
      [string compare "on" $value] == 0 ||
      [string compare "true" $value] == 0 ||
    $value == "1"} {
    return 1
  } elseif {[string compare "no" $value] == 0 ||
            [string compare "off" $value] == 0 ||
            [string compare "false" $value] == 0 ||
            $value == "0"} {
    return 0
  }
  return $default
}

proc readconfig {} {
  global userpass
  global source_dir
  global http_port
  global url_prefix
  global description_hover
  global multi_delete
  global tyshow_links
  global ethernet_interface
  global filter
  global description_show
  global lan_auth
  global rss_auth
  global hosts_allow
  global allow_merge
  global show_tivor
  global hide_ppv
  global hide_not_received
  global hide_xm
  global dst_policy
  global sf_stats
  global black_nums
  global black_calls
  global black_names

  set user ""
  set pass ""
  set http_port 80
  set url_prefix ""
  set default_theme "technophobe"
  set description_hover 1
  set multi_delete 1
  set tyshow_links 0
  set ethernet_interface ""
  set filter ""
  set description_show 0
  set lan_auth 0
  set rss_auth 0
  set hosts_allow ""
  set allow_merge 0
  set show_tivor 0
  set hide_ppv 1
  set hide_not_received 0
  set hide_xm 0
  set dst_policy 0
  set sf_stats 1
  if {[catch {set fd [open "$source_dir/tivoweb.cfg" "r"]}]} {
    puts "Error opening configuration file 'tivoweb.cfg'"
  } else {
    set line [gets $fd]
    while { ![eof $fd] } {
      if {[regexp -nocase {^([^ #]+) *= *(.*) *$} $line dummy varname value] == 1} {
        set varname [string tolower $varname]
        set tvalue [string trim $value "\"'"]
        if {[string compare "username" $varname] == 0} {
          set user $tvalue
        } elseif {[string compare "password" $varname] == 0} {
          set pass $tvalue
        } elseif {[string compare "port" $varname] == 0} {
          set http_port $tvalue
        } elseif {[string compare "ethernetinterface" $varname] == 0} {
          set ethernet_interface $tvalue
        } elseif {[string compare "filter" $varname] == 0} {
          set filter $tvalue
        } elseif {[string compare "prefix" $varname] == 0} {
          set url_prefix $tvalue
          if {[string range $url_prefix end end] != "/"} {
            append url_prefix "/"
          }
          if {[string index $url_prefix 0] == "/"} {
            set url_prefix [string range $url_prefix 1 end]
          }
        } elseif {[string compare "theme" $varname] == 0} {
          regsub {\.css$} $tvalue {} tvalue
          if { $tvalue != "" } {
            if { [catch { set fd2 [open "$source_dir/themes/$tvalue.css" "r"] }] == 0 } {
              set default_theme $tvalue
              close $fd2
            }
          }
        } elseif {[string compare "descriptionhover" $varname] == 0} {
          set description_hover [binary_compare $tvalue 1]
        } elseif {[string compare "multidelete" $varname] == 0} {
          set multi_delete [binary_compare $tvalue 1]
        } elseif {[string compare "tyshowlinks" $varname] == 0} {
          set tyshow_links [binary_compare $tvalue 0]
        } elseif {[string compare "descriptionshow" $varname] == 0} {
          set description_show [binary_compare $tvalue 1]
        } elseif {[string compare "lan_auth" $varname] == 0} {
          set lan_auth [binary_compare $tvalue 1]
        } elseif {[string compare "rss_auth" $varname] == 0} {
          set rss_auth [binary_compare $tvalue 1]
        } elseif {[string compare "hostsallow" $varname] == 0} {
          set hosts_allow $tvalue
        } elseif {[string compare "dst_policy" $varname] == 0} {
          set dst_policy $tvalue
        } elseif {[string compare "sf_stats" $varname] == 0} {
          set sf_stats $tvalue
        } elseif {[string compare "showmergebutton" $varname] == 0} {
          set allow_merge [binary_compare $tvalue 0]
        } elseif {[string compare "showtivorentries" $varname] == 0} {
          set show_tivor [binary_compare $tvalue 0]
        } elseif {[string compare "hideppv" $varname] == 0} {
          set hide_ppv [binary_compare $tvalue 1]
          #puts "setting HidePPV $hide_ppv"
        } elseif {[string compare "hidenotreceived" $varname] == 0} {
          set hide_not_received [binary_compare $tvalue 1]
          #puts "setting HideNotReceived $hide_not_received"
        } elseif {[string compare "hidexm" $varname] == 0} {
          set hide_xm [binary_compare $tvalue 1]
          #puts "setting HideXM $hide_xm"
        } elseif {[string compare "blacknums" $varname] == 0} {
          set black_nums $value
        } elseif {[string compare "blackcalls" $varname] == 0} {
          set black_calls $value
        } elseif {[string compare "blacknames" $varname] == 0} {
          set black_names $value
        } else {
          puts "Config option not recognized: '$line'"
        }
      }
      set line [gets $fd]
    }
    close $fd
  }

  if {[string length $user] > 0} {
    set userpass "$user:$pass"
  } else {
    set userpass ""
  }
  if {$TWP::STYLE == "" } {
    set TWP::STYLE $default_theme
  }
}

proc loadfile {filename} {
  if {![file exists $filename]} {
    set openfile [open $filename w]
    close $openfile
  }
  set openfile [open $filename r]
  while {[gets $openfile line] >= 0} {
    set ret [lappend ret $line]
  }
  close $openfile
  return $ret
}

proc savefile {filename list} {
  set writefile [open $filename w]
  foreach element $list {
    puts $writefile $element
  }
  close $writefile
  return $list
}

proc updateconfig {label value} {
  global source_dir
  set cfg [loadfile $source_dir/tivoweb.cfg]
  set pos [lsearch -regexp $cfg "^$label *="]
  if {$pos >= 0} {
    set cfg [lreplace $cfg $pos $pos "$label = $value"]
  } else {
    lappend cfg "$label = $value"
  }
  savefile $source_dir/tivoweb.cfg $cfg
}

# proc for displaying time using timezone offset
proc get_tzoffset { time } {
  global uktivo
  global stzoffset
  global daylightsavings
  set offset $stzoffset
  if { $time == ""} {
    set time 0
  }
  incr time $offset
  if { $daylightsavings == 2 } {
    set date [clock format $time -format "%1d %w %1m %1H %1M"]
    scan $date "%d %d %d %d %d" dom dow month hour min
    set dlsval 0
    if {$uktivo} {
      if {$month > 3 && $month < 10} {
        set dlsval 1
      } elseif {$month == 3 && $dom >= 25 && $dow == 0 && $hour >= 1} {
        set dlsval 1
      } elseif {$month == 3 && $dom >= 25 && $dow != 0 && ($dom-24-$dow >= 1) } {
        set dlsval 1
      }
    } else {
      if {$month > 4 && $month < 10} {
        set dlsval 1
      } elseif {$month == 4 && $dom > 7} {
        set dlsval 1
      } elseif {$month == 4 && $dom <= 7 && $dow == 0 && $hour >= 2} {
        set dlsval 1
      } elseif {$month == 4 && $dom <= 7 && $dow != 0 && ($dom-$dow > 0)} {
        set dlsval 1
      }
    }   
    if {$dlsval == 0} {
      if {$month == 10 && $dom < 25} {
        set dlsval 1
      } elseif {$month == 10 && $dom >= 25 && $dow == 0 && $hour < 1 && !$uktivo} {
        set dlsval 1
      } elseif {$month == 10 && $dom >= 25 && $dow != 0 && ($dom-24-$dow < 1) } {
        set dlsval 1
      }
    }   
    if {$dlsval} {
      incr offset 3600
    }
  }
  return $offset
}

proc ftime { time format } {
  return [clock format [expr $time + [get_tzoffset $time]] -format $format]
}

global db
global startuptime
global module_list
global reload
global tivoswversion
global content_suffix_list
global content_handler_list
global version3
global dtivo
global sn
global uktivo
global ip_address
global series1
global version
global bin_dir
global num_tuners
global lang

set dbPoolSize [expr 300 * 1024]
set startuptime [clock seconds]
set db [dbopen $dbPoolSize]
set module_list ""
set content_list ""
set content_suffix_list ""
set content_handler_list ""
set reload 0
set dtivo [MfsFileExists /ApgBoot]

RetryTransaction {

  if { [info exists env(SerialNumber)] && [string trim $env(SerialNumber) "0"] != "" && [string length $env(SerialNumber)] == 15 } {
    set sn $env(SerialNumber)
  } else {
    set sn [exec crypto -gsn]
  }
  ## get service number

  set model [string index $sn 0]
  ## model; 0 = Series 1, 1-3 = Series 2

  set manuf [string index $sn 1]
  ## manufacturer; 0 = Philips, 1 = Sony, 2 = Thompson/RCA, 3 = Hughes/AT&T, 4 = TiVo, 5 = Hughes, 6 = Toshiba, 7 = Pioneer, 8 = Samsung
  ## for those with two manufacturers, the first is Series 1 and the second is Series 2

  set opt [string index $sn 2]
  ## type; 0,2 = StandAlone, 1 = DirecTiVo, 3 = UK/PAL, 4 = DVD, 5 = DVD Writer
  
  set swsystem [db $db open /SwSystem/ACTIVE]
  set tivoswversion [dbobj $swsystem get Name]

  set version [string index $tivoswversion 0]
  set suffix [string index $tivoswversion [expr [string length $tivoswversion] - 5]]
  
  set uktivo [expr [string index $tivoswversion [expr [string length $tivoswversion] - 1]] == 3]
  if { $uktivo || [file exists /etc/oztivo-version] || [file exists /etc/oztivo_version] } {
    set lang "en"
  } else {
    set lang "us"
  } 

  if { $suffix == 2 } {
    set series1 0
    set bin_dir $source_dir/bin_mips
  } else {
    set series1 1
    set bin_dir $source_dir/bin_ppc
  }
  
  if {$::version >= 3} {
    set version3 1
    set lconfig  [db $db open /State/LocationConfig]
    #set setup [db $db open /State/ServiceConfig]
    if { $::version >=4 } {
      set setuptz [dbobj $lconfig get TimeZoneOffset]
    } else {
      set setuptz [dbobj $lconfig get TimeZoneOld]
    }
    if { $::version >=5 } {
      # Assumes that UseDaylightSavings uses 0=No, 1=Yes, NA=Yes
      set daylightsavings [dbobj $lconfig get UseDaylightSavings]
      if {$daylightsavings != ""} {incr daylightsavings}
    } else {
      # DaylightSavingsPolicy uses 1=No, 2=Yes
      set daylightsavings [dbobj $lconfig get DaylightSavingsPolicy]
    }
    
    set AvConfig [db $db open /State/AvConfig]
  } else {
    set version3 0
    set setup [db $db open /Setup]
    set setuptz [dbobj $setup get TimeZone]
    set daylightsavings [dbobj $setup get DaylightSavingsPolicy]
    set AvConfig [db $db open /Setup]
  }

  set num_tuners [defaultval 1 [dbobj $AvConfig get TunerCount]]
}

if {$setuptz == ""} { set setuptz 0 }
if {$daylightsavings == ""} { set daylightsavings 2 }

# NEW DST
if { $setuptz <= 0 || $version >= 4} {
  set tz $setuptz
} else {
  set tzlist "-5 -6 -7 -8 -9 -10 0 1 2 3 4 5 6 7 8 9 10 11 12 -1 -2 -3 -4 -11 -12"
  set tz [expr [lindex $tzlist [expr $setuptz - 1]]*60*60]
  unset tzlist
}
set stzoffset [expr round(([clock scan [lreplace [exec date -u] 4 4]] - [clock seconds] + $tz)/60.0)*60]

# OLD DST
set tzoffset [expr round(([clock scan [lreplace [exec date -u] 4 4]] - [clock seconds] + [get_tzoffset [clock seconds]])/60.0)*60]

register_content_handler "js" "text/javascript" "$source_dir/scripts" 0 serve_file
register_content_handler "css" "text/css" "$source_dir/themes" 0 serve_file
register_content_handler "txt" "text/plain" "$source_dir" 0 serve_file
register_content_handler "png" "image/png" "$source_dir/images" 1 serve_image
register_content_handler "gif" "image/gif" "$source_dir/images" 1 serve_image
register_content_handler "jpg" "image/jpg" "$source_dir/images/Gallery" 1 serve_image

proc serve_asx {chan fsid head_req last_modified} {
  global db

  if {[regexp -nocase {([^.]*)\.asx} $fsid dummy rest]} {
    set fsid $rest
  }
  if {[string index $fsid 0] == "/"} {
    set fsid [string range $fsid 1 end]
  }

  set title ""

  catch {
    RetryTransaction {
      set rec [db $db openid $fsid]
      set showing [dbobj $rec get Showing]
      set program [dbobj $showing get Program]
      set title [strim [dbobj $program get Title]]
      set isepisode [dbobj $program get IsEpisode]
      if { $isepisode == 1 } {
        set eptitle [strim [dbobj $program get EpisodeTitle]]
      } else {
        set eptitle ""
      }
    }
  }
  if { $title != "" } {
    print_html_header_200 $chan "video/x-ms-asx" $last_modified
    puts $chan "<asx version=\"3.0\">"
    puts $chan "<title>TiVoWeb ASX file</title>"
    puts $chan "<entry>"
    puts $chan "<title>$title - $eptitle</title>"
    puts $chan "<author>TiVo Stream</author>"
    puts $chan "<ref href=\"tivo://$::ip_address/$fsid\" />"
    puts $chan "</entry>"
    puts $chan "</asx>"
  } else {
    print_html_header_404 $chan
  }
}

register_content_handler "asx" "video/x-ms-asx" "$source_dir" 0 serve_asx

set sock ""

proc vwait_new v {
  upvar $v x
  set t $x
  while {$x == $t} {
    after 100
    update
  }
}

mySource $source_dir/libs/html.itcl

if {$::version <= 4} {
  mySource $source_dir/libs/sendkey.itcl
  unset evrc
}

while { $reload != "quit" } {
  readconfig

  if {$::dst_policy != 0} {
    # Override settings
    set daylightsavings $::dst_policy
  }

  set timer 45
  while {$timer > 0} {
    if { $ethernet_interface == "" } {
      set ip_address [lindex [split [string trim [exec ifconfig | grep "inet addr" | grep -v "127.0.0.1"]] " :"] 2]
    } else {
      set ip_address [lindex [split [string trim [exec ifconfig $ethernet_interface | grep "inet addr" ]] " :"] 2]
    }
    if { $ip_address == "" } {
      puts "Waiting for network interface"
      after 3000
      set timer [expr $timer - 3]
    } else {
      break
    }
  }
  if { $ip_address == "" } {
    puts "ERROR: Unable to ascertain network IP Address"
    exit 1
  }
  set hostname $env(HOSTNAME)
  if {$hostname == "(none)"} {
    set hostname $ip_address
  }

  if { $sock != "" } { close $sock }
  set sock [socket -server ::session $http_port]

  #set tcl_traceExec 1

  puts "Loading modules..."
  set module_list ""
  set modules [glob "$source_dir/modules/*.itcl"]
  set modules [lsort $modules]
  foreach module $modules {
    set errorCode ""
    set errorInfo ""
    puts [file rootname [file tail $module]]
    mySource $module
  }
  unset module
  unset modules
  make_menu

  set reload -1
  puts "Accepting Connections"
  if {$::version == 4} {
    # DaveA 8/16/05: BTUx9 reports that vwait is broken on software
    # version 4: when the select is called it is passed a wait param
    # of 0, which returns immediately... effectively putting the
    # process in a very tight polling loop that jumps back and forth to
    # kernel space.  This causes any tcl script to use all available CPU.
    vwait_new reload
  } else {
    vwait reload
  }
}

puts "Web Server Terminated"
