#!/tvbin/tivosh
# $Id$
# changes for v1.4 --btux9
#   split out most of the guts to load.tcl so reloads will be more effective
#   added support of /var/twpfailed file to prevent reboot loops
#   added dyncfg support
# Tries to open port 8080 if default fails --btux9

proc load {level {restart 0}} {
  global reload source_dir
  set reload $level
  source $source_dir/load.tcl
}

catch {EnableTransactionHoldoff true}
set mfsVerboseG 0
source $tcl_library/tv/log.tcl
source $tcl_library/tv/mfslib.tcl
source $tcl_library/tv/dumpobj.tcl
#~ set ::source_dir [file dirname [info script]]
cd [file dirname [info script]]
set ::source_dir [pwd]
# disable keyname echoing to controling tty
set ::verboseSendKeyG 0
#~ set ::dbPoolSize [expr 300 * 1024]
set ::db [dbopen [expr 300 * 1024]]
set r [expr {$argv=="restart"}]
#~ puts argv=$argv

set ::last_load 0
load 0

set port $::conf(Port)
while 1 {
  if { $::conf(EthernetInterface) != "" } {
    puts "Attempting to open port $port on $ip_address"
    if ![catch {set sock [socket -server ::session -myaddr $ip_address $port]} errs] break
  } else {
    puts "Attempting to open port $port"
    if ![catch {set sock [socket -server ::session $port]} errs] break
  }
  if {$port==8080} {error "Failed to open socket... aborting"}
  puts "Couldn't open port $port (there may be a server already running), trying 8080"
  #~ set sock [socket -server ::session 8080]
  set port 8080
}
puts "Accepting Connections"

# Delete /var/twpfailed after a minute to indicate that we successfully loaded
after 60000 {file delete /var/twpfailed}

if {$::version == 4} {
  # DaveA 8/16/05: BTUx9 reports that vwait is broken on software
  # version 4: when the select is called it is passed a wait param
  # of 0, which returns immediately... effectively putting the
  # process in a very tight polling loop that jumps back and forth to
  # kernel space.  This causes any tcl script to use all available CPU.
  # modified to be a more generic vwait replacement -- btux9
  puts "Using modified vwait"
  rename vwait vwait_old
  proc vwait {v args} {
    set vloc ::vwait_[string trimleft $v ":"]
    if {[llength $args]==2} {
      set $vloc 1
      return
    }
    if [info exists $vloc] { unset $vloc }
    upvar $v waitvar
    trace variable waitvar w vwait
    while {![info exists $vloc]} {
      after 100
      update
    }
  }
}

while 1 {
  #dyncfg has changed... save changes
  saveConfig dyncfg "array set ::dyncfg \{\n[array get ::dyncfg]\n\}"
  vwait dyncfg
}
