## init.tcl -BTUx9
# moved "full reload" code here (from load.tcl) because:
#     1) it'll run in MOD:: and be cleaned up automagically
#     2) code folding works much better with code that's separated into procs, and
#     3) the code doesn't have to be read for quick reloads

variable init_loaded 1

proc ::mySource {f {ns "MOD"}} {
# mySource sources a file and strips comments and indents... saves LOTS of space
  set fd [open $f r]
  set s [read $fd]
  close $fd
  regsub -all "\n(\[ \t]+|\n|#\[^\n]*\n)+" $s "\n" s
  set ns ::$ns
  set ::NS $ns
  set ret [catch {namespace eval $ns "$s"}]
  if {$ret==1} {
    puts "$::errorCode $::errorInfo"
  }
}

proc ::defdbget {obj field default} { return [expr {[set r [dbobj $obj get $field]]==""?$default:$r}] }

proc init_globals {} {
# Determine data dir, create subdirs if necessary
  global env data_dir
  if [info exists env(TWP_DATA_DIR)] { set data_dir $env(TWP_DATA_DIR) } else { set data_dir $::source_dir }
  foreach d {"" config backups} {
    if {![file exists $data_dir/$d]} {file mkdir $data_dir/$d}
  }

  #~ namespace eval ::TWP {
  global TWP
  array set TWP [list NAME	"TivoWebPlus" VERSION "2.1.b3" OPTIONS_MENU "" SUB_MENU ""]
  set TWP(DIST_VERSION) "$TWP(VERSION) CVS"
  if {[file exists $::source_dir/.dist/version]} {
    set v [string trim [exec cat $::source_dir/.dist/version] \n]
    if [PrefixMatches $TWP(VERSION) $v] { set TWP(DIST_VERSION) $v }
  }
  puts "$TWP(NAME) - v$TWP(VERSION) $TWP(DIST_VERSION)\nThe program comes with ABSOLUTELY NO WARRANTY.\nThis program is licensed under the GPL.\nSee the 'docs/copyright' and 'docs/index.html' files for copyright and credit information."
  #~ }

  global tr_rowcount dtivo sn
  catch {unset ::hookA}
  set tr_rowcount 0
# added secondary test for dtivo (for unsubbed boxes) --btux9
  set dtivo [expr [MfsFileExists /ApgBoot] || [MfsFileExists /Resource/LoopSet/DirecTV_Blue_1]]
  if {[info exists env(SerialNumber)] && [string trim $env(SerialNumber) "0"]!="" && [string length $env(SerialNumber)]==15} {
    set sn $env(SerialNumber)
  } else {
    set sn [exec crypto -gsn]
    set env(SerialNumber) $sn
  }
  #~ ## model; 0 = Series 1, 1-3 = Series 2
  #~ set model [string index $sn 0]
  #~ ## manufacturer; 0 = Philips, 1 = Sony, 2 = Thompson/RCA, 3 = Hughes/AT&T, 4 = TiVo, 5 = Hughes, 6 = Toshiba, 7 = Pioneer, 8 = Samsung
  #~ ## for those with two manufacturers, the first is Series 1 and the second is Series 2
  #~ set manuf [string index $sn 1]
  #~ ## type; 0,2 = StandAlone, 1 = DirecTiVo, 3 = UK/PAL, 4 = DVD, 5 = DVD Writer
  #~ set opt [string index $sn 2]

  global tivoswversion version version3 guideindexdir seasonpassdir nowshowingdir num_tuners setupid defrecquality datagroup
  RetryTransaction {
    if [catch {mfs find /State/TWP}] { mfs mkdir /State/TWP }
    set swsystem [db $::db open /SwSystem/ACTIVE]
    set tivoswversion [dbobj $swsystem get Name]
    set version [lindex [split $tivoswversion .] 0]
    if {$::version >= 3} {
      set version3 1
      set AvConfig [db $::db open /State/AvConfig]
      if {$version >= 6 || [PrefixMatches 3.5 $tivoswversion]} {
        set guideindexdir "/GuideIndexV3"
      } else {
        set guideindexdir "/GuideIndexV2"
      }
      set seasonpassdir "/SeasonPass/User"
      set nowshowingdir "/Recording/NowShowingByClassic"
      set service [db $::db open /State/ServiceConfig]
      set datagroup [dbobj $service get DataGroupList]
    } else {
      set version3 0
      set AvConfig [db $::db open /Setup]
      set guideindexdir "/GuideIndex"
      set seasonpassdir "/SeasonPass"
      set nowshowingdir "/Recording/NowShowing"
      set service [dbobj $AvConfig get ServiceInfo]
      set datagroup [dbobj $service get DataGroupList]
    }
    set defrecquality [defdbget $AvConfig RecordQuality 100]
    set num_tuners [defdbget $AvConfig TunerCount 1]
    set setup [db $::db open /Setup]
    set setupid [dbobj $setup fsid]
  }

  global series1 bin_dir uktivo lang langa
  set suffix [string index $tivoswversion [expr [string length $tivoswversion] - 5]]
  if { $suffix == 2 } {
    set series1 0
    set bin_dir $::source_dir/bin_mips
  } else {
    set series1 1
    set bin_dir $::source_dir/bin_ppc
  }
  set uktivo [expr [string index $tivoswversion [expr [string length $tivoswversion] - 1]] == 3]
  if { $uktivo || [file exists /etc/oztivo-version] || [file exists /etc/oztivo_version] } {
    set lang "en"
    set langa {favourite colour}
  } else {
    set lang "us"
    set langa {favorite color}
  }

  global httpActive
  set httpActive 0
  foreach v {cronFuncs cronPeriods cronCurr} {
    global $v
    set $v {}
  }

  global selectiontypes presentationbehtypes states showtypes premieretypes
  set selectiontypes {{Show Recommendation} {Package Recommendation} {By Name} {By Channel} Manual {Suggestion} {Fuzzy Package} Bookmark {Season Pass} {Manual Season Pass} Guide IPreview WishListPass Extended ManualExtended CaptureRequest DailyData}
  set presentationbehtypes {Single {Season Pass} Wishlist {Watch Only} {Tivo Clips} Suggestion {Tivo Clips Suggestion} Invisible {Manual Season Pass} Manual {Cache History}}
  set states {{To Do} Cancelled {In Progress} {Now Showing} Deleted {To Do} Allocated}
  set showtypes {Serial {Short Film} Special {Limited Series} Series Miniseries {Paid Programming}}
  set premieretypes {{Season Finale} {Series Finale} Premiere {Season Premiere} {Series Premiere}}

  global images
  if { $version == 3 } { set n 12
  } elseif { [PrefixMatches "2.5.5" $tivoswversion] } { set n 10
  } else { set n 9 }
  set images "ExpireNever-256.8.png SelectIcon-256.$n.png ThumbUp2-256.8.png ExpireSoon-256.8.png ThumbDn1-256.8.png Expired-256.8.png ThumbDn2-256.8.png ThumbUp3-256.9.png Recording-256.9.png ThumbDn3-256.8.png TivoSuggest-256.8.png SeasonPass.9.png ThumbUp1-256.8.png WishListPass.2.png LargeSquarePredicted3DownThumbs.3.png LargeSquarePredicted2DownThumbs.3.png LargeSquarePredicted1DownThumbs.3.png LargeSquarePredicted1UpThumbs.3.png LargeSquarePredicted2UpThumbs.3.png LargeSquarePredicted3UpThumbs.3.png recycle1.png PaidSuggest-256.3.png"

  global tvratingnums tvratingvals tvratingimages showingbitnums showingbitvals advisorynums advisoryvals mpaaratingnums mpaaratingvals mpaaratingimages
##TODO - clean up use of these nums , vals, and images
  #if { [PrefixMatches "2.5.1" $tivoswversion] } {
  #   set tvratings [GetResourceData 851968 851993 851998 1 0]
  #} elseif { [PrefixMatches "2.0.1" $tivoswversion] } {
  #   set tvratings [GetResourceData 983040 983065 983070 1 0]
  #}
  #set tvratingnums [lindex $tvratings 0]
  #set tvratingvals [lindex $tvratings 1]
  set tvratingnums "1 2 3 4 5 6"
  set tvratingvals "TV-Y7 TV-Y TV-G TV-PG TV-14 TV-MA"
  set tvratingimages "tvy7.png tvy.png tvg.png tvpg.png tv14.png tvma.png"

  #if { [PrefixMatches "2.5.1" $tivoswversion] } {
  #   set showingbits [GetResourceData 1441792 1441843 1441863 0 1]
  #} elseif { [PrefixMatches "2.0.1" $tivoswversion] } {
  #   set showingbits [GetResourceData 1572864 1572915 1572935 0 1]
  #}
  #set showingbitnums [lindex $showingbits 0]
  #set showingbitvals [lindex $showingbits 1]
  set showingbitnums "1 2 4 8 16 32 64 128 256 512 1024 65536 131072 262144 524288 1048576"
  set showingbitvals "CC Stereo Sub {In Prog} Class SAP Blackout Intercast {Three D} R Letterbox S V L D FV"

  #if { [PrefixMatches "2.5.1" $tivoswversion] } {
  #   set advisorys [GetResourceData 917504 917533 917542 1 0]
  #} elseif { [PrefixMatches "2.0.1" $tivoswversion] } {
  #   set advisorys [GetResourceData 1048576 1048605 1048614 1 0]
  #}
  #set advisorynums [lindex $advisorys 0]
  #set advisoryvals [lindex $advisorys 1]
  set advisorynums "1 2 3 4 5 6 7 8 9 10"
  set advisoryvals "AL GL N BN GV V MV SC RP AC"

  #set mpaaratings [GetResourceData 65536 65567 65578 1 0]
  #set mpaaratingnums [lindex $mpaaratings 0]
  #set mpaaratingvals [lindex $mpaaratings 1]
  set mpaaratingnums "1 2 3 4 5 6 7 8 9 10 11 12"
  set mpaaratingvals "G PG PG-13 R X NC-17 AO {Rated NR} U 12 15 18"
  set mpaaratingimages "MPAA_G.png MPAA_PG.png MPAA_PG-13.png MPAA_R.png MPAA_NC-17.png MPAA_AO.png MPAA_X.png MPAA_NR.png"

  global 404_counter INSULTS
  set 404_counter 0
  set INSULTS { ""
    {I don't wanna talk to you no more, you empty headed animal food trough wiper! I fart in your general direction! Your mother was a hamster and your father smelt of elderberries! Now go away or I shall taunt you a second time-a!}
    {Here I am, brain the size of a tivo and you ask me to display a 404 page? Call that job satisfaction? 'Cos I sure don't!}
    {It's about to print out a 404 error. I can tell by the intolerable air of smugness it suddenly generates. It's tivoweb's pleasure to render a page for you and their satisfaction to close the socket afterwards with the knowledge of a job well done. Thank you tivoweb programmers, 'let's not write new features; let's waste a ton of memory on 404 messages' they said. I'm a 404 prototype. You can tell can't you? I hate 404 messages. I'm not getting you down am I?}
    {Oh look, it's another one of those mindbogglingly useful 404 pages. Don't pretend that you wanted to see this page, I know you hate me, everybody does. It's part of the shape of the universe. I only have to talk to somebody and they begin to hate me. Even other webservers hate me. If you just ignore me I expect I shall probably go away.}
    {The page you are looking for is currently on display in the bottom of a locked filing cabinet stuck in a disused lavatory with a sign on the door that says: 'Beware of Leopard'}
    {*&^@%!$%! where'd I put that page?}
    {
<pre>
Tmk Fatal Error: Thread tivoweb &lt;42&gt; died due to signal -2
NIP 0x1a36728 link 0x1c485ac ctr 29802260
R00 0x00000025  R01 0x7ffff3e0  R02 0x00000100  R03 0x00000000
R04 0x80a36018  R05 0x3003d5d7  R06 0x00000000  R07 0x00000000
R08 0x19999999  R09 0x01d88e6c  R10 0x000000e6  R11 0x3003d5d7
R12 0x00000000  R13 0x01d9ac08  R14 0x30000fb4  R15 0x00000000
R16 0x00000000  R17 0x00000000  R18 0x30000fc8  R19 0xffffffff
R20 0x30041a2e  R21 0x00000000  R22 0x3001839c  R23 0x00000002
R24 0x00000002  R25 0x00000003  R26 0x00000003  R27 0x00000004
R28 0x30015f40  R29 0x00000002  R30 0x30000fd0  R31 0x00000000
1a29e08 7ffff2f8 1c48598 1c48118 1c41c18 1c6c284 1c55e18 1c72934 1c7a520 1c41650 1c1d4e4 18
</pre>(just kidding)}
    {You're rather presumptuous aren't you? I merely start to display the page and I'm immediately harrased by you before given time to do so. Perhaps I was actually going to give you the page this time... farell, have a good evening of being unfriendly}
    {Ni! Ni! Ni! Ni! We shall say Ni! again if you do not appease us}
    {
&lt;\[mbm\]&gt; hmm, why doesn't tivoweb have some lame ass easter egg ?<br>
&lt;lightn&gt; I thought your idea of insulting 404 errors was good<br>
&lt;\[mbm\]&gt; thought you turned that down, something about censorship<br>
&lt;lightn&gt; huh? no, I always liked it, I just didn't like your 'click here to crash your tivo'
      }
    {Surely you've not given up your quest for the Holy Grail?}
    {I'm not quite dead yet!}
    {Run away! Run away! Run away! Run away! Run away! Run away! Run away!}
    {O Lord, bless this Thy tivoweb server that, with it, Thou mayest blow Thine enemies to tiny bits in Thy mercy}
    {Now it is such a bizarrely improbable coincidence that anything so mindboggingly useful as this 404 page could have evolved purely by chance that some thinkers have chosen to see it as the final and clinching proof of the non-existance of tivoweb programmers.
<p>
The argument goes something like this: `I refuse to prove that we exist,' said one tivoweb programmer, 'for proof denies deniability and without deniability there'd be bug reports.'
<p>'But,' says Man, 'Surely the 404 page is a dead giveaway isn't it? It could not have evolved by chance. It proves you exist, and so therefore, by your own arguments, you don't. QED.'
<p>'Oh dear,' said the tivoweb programmer, 'I hadn't thought of that,' and the tivoweb programmers promptly vanished under a flood of bug reports.}
    {I knew I should have taken that left in Albuquerque!}
    {YOUR AD HERE.}
    {This Space for Rent}
    {This page intentionally left blank.}
  }

  global html_enc
  set html_enc(mask) {a-zA-Z0-9}
  set i 256
  while {$i} {
    incr i -1
    set c [format %c $i]
    if {![string match \[$html_enc(mask)\] $c]} {
      set html_enc($c) %[format %02X $i]
    }
  }
  #~ array set html_enc {& &amp; < &lt; > &gt;}

  global html_dec
  set html_dec(mask) {a-zA-Z0-9}
  set i 256
  while {$i} {
    incr i -1
    set c [format %c $i]
    if {![string match \[$html_dec(mask)\] $c]} {
      set html_dec(%[format %02X $i]) $c
      set html_dec(%[format %02x $i]) $c
    }
  }
  array set html_dec {\+ " " \[ \[ \{ \{ $ $ & \001 ; \001 = \002 \\ \\ \} \}}

  global buckmask
  set buckmask [lindex {{id fname invdate invtime vis xid type inProgress} {fname id invdate invtime xid type inProgress}} [expr $::version<9]]

  global UC
  foreach l [split "abcdefghijklmnopqrstuvwxyz" ""] {set UC($l) [string toupper $l]}

}

proc GetResourceData {refid startid endid startindex bits} {
  #~ foreach  $x break
  set keys ""
  set vals ""
  RetryTransaction {
    set sws [db $::db open /SwSystem/ACTIVE]
    set offset [expr $refid/65536 - 1]
    set resource [dbobj $sws get ResourceGroup $offset]
    set id [dbobj $resource get Id]
    if { $id == $refid } {
      set items [dbobj $resource get Item]
      foreach item $items {
        set id [dbobj $item get Id]
        if { $id >= $startid && $id <= $endid } {
          if { $bits == 1 } {
            set value [expr 1 << ($id - $startid + $startindex)]
          } else {
            set value [expr $id - $startid + $startindex]
          }
          set strval [strim [dbobj $item get String]]
          if { $strval != "" } {
            lappend keys $value
            lappend vals $strval
          }
        }
      }
    } else {
      error "Error: refid didn't match"
    }
  }
  return [list $keys $vals]
}

proc getDTivoGenres {} {
  set cat {}
  set keys ""
  RetryTransaction {
    set apgboot [db $::db open /ApgBoot]
    set categorysystem [dbobj $apgboot get RootCategorySystem]
    set labels [dbobj $categorysystem gettarget Labels]
  }
  ForeachTrans id $labels 50 {
    set clabel [openid $id]
    set index [dbobj $clabel get Index]
    #set fcoreset [dbobj $clabel get FCoreSet]
    if {$index != 0} {
      set vlabel [strim [dbobj $clabel get Label]]
      lappend cat [llength $keys]
      lappend keys $index
      lappend vals $vlabel
    }
    set subcat [dbobj $clabel get Subcategory]
    if {$subcat != ""} {
      set labels2 [dbobj $subcat get Labels]
      foreach clabel2 $labels2 {
        set index [dbobj $clabel2 get Index]
        #set fcoreset [dbobj $clabel2 get FCoreSet]
        if {$index != 0} {
          set vlabel [strim [dbobj $clabel2 get Label]]
          #~ if {[lsearch $keys $index]>-1} {
            #~ puts $vlabel
          #~ }
          lappend keys $index
          lappend vals $vlabel
        }
      }
    }
  }
  lappend cat [llength $keys]
  return [list $keys $vals $cat]
}

proc getSAGenres {} {
  ForeachMfsFile id index type "/Genre" "" {
    if {$index<1000} { set ids($id) $index } else { lappend icats $id $index }
  }
  if ![info exists ids] {return ""}
  #
  # Under some systems (ie UK v2.5.5) there are sometimes many
  # children (ie for the Sport genre) and the "get Child" command
  # will open up each as an object... the problem is that if more
  # than about 400 actions are performed within a RetryTransaction
  # then a "CONFLICT err=0x00030019" error will result.  The only
  # way around this is to split up the transactions into smaller
  # pieces...
  #
  set keys {}
  set vals {}
  set cats {}
  set incrsize 50
  foreach {id index} $icats {
    set done 0
    set idx 0
    while {!$done} {
      RetryTransaction {
        set obj [openid $id]
        set name [dbobj $obj get Name]
        if {$idx == 0} {
          lappend cats [llength $keys]
          lappend keys $index
          lappend vals [strim $name]
        }
        set fsids [dbobj $obj gettarget Child]
        set num [llength $fsids]
        if {$fsids != ""} {
          set max [expr ($num > $idx + $incrsize) ? ($idx + $incrsize) : $num]
          for {set i $idx} {$i < $max} {incr i} {
            set child [dbobj $obj get       Child $i]
            set fsid  [dbobj $obj gettarget Child $i]
            set fsid [lindex [split $fsid /] 0]
            lappend keys $ids($fsid)
            lappend vals [strim [dbobj $child get Name]]
          }
          if {$num == $max} {
            set done 1
          } else {
            incr idx $incrsize
          }
        } else {
          set done 1
        }
      }
    }
  }
  lappend cats [llength $keys]
  return [list $keys $vals $cats]
}

proc getGenres {} {
  global genrenums genrevals genrecats
  if [info exists genrenums] return
  if {$::dtivo} {
    if [catch {set genres [getDTivoGenres]} ] { set genres {{} {} {}} }
  } else {
    if {[set genres [getSAGenres]]==""} {
      switch -glob $::tivoswversion {
        3.0* { set gix 1 }
        2.5* { set gix 2 }
        2.0* { set gix 3 }
        default { set gix 0 }
      }
      ##todo - if this throws an error, need to supply genre data from elsewhere
      catch { set genres [GetResourceData [lindex {{655360 655377 655495 1 0} {655360 655377 655495 1 0} {655360 655377 655495 1 0} {720896 720913 721031 1 0}} $gix]] }
      ##todo - figure out how to get genreCats for SA genres
      lappend genres {}
    }
  }
  foreach {genrenums genrevals genrecats} $genres break
  #~ set genrecats [lindex $genres 2]
}

proc readconfig {} {
  global dyncfg userpass conf
# populate dyncfg array
  loadConfig dyncfg 1
  set dver $dyncfg(version)
  set newver 2
  if {$dver>$newver} {
    puts "Warning: Downgrade detected... clearing dyncfg"
    file rename -force $::data_dir/config/dyncfg.cfg $::data_dir/backups/.
    loadConfig dyncfg 1
  } elseif {$dver<$newver} {
    if {$dver<1} {
      foreach {f v} [array get ::dyncfg] {
        if [PrefixMatches "Fmt_" $f] {
          unset ::dyncfg($f)
        }
      }
    }
    if [info exists dyncfg(folders)] {
      set f $dyncfg(folders)
      unset dyncfg(folders)
      set m [lrange $dyncfg(menus) 0 1]
      set l [llength $f]
      foreach {s l mods} $f {
        if {[lsearch {Invisible Main MainMenu} $s]>=0} continue
        lappend m [list $s $l $mods]
      }
      unset mods
      set dyncfg(menus) $m
    }
    foreach m {menu_sort sortgroups groupsort} { catch {unset dyncfg($m)} }
    # update version
    set dyncfg(version) $newver
    #~ saveConfig dyncfg "array set ::dyncfg \{\n[array get dyncfg]\n\}"
    #~ loadConfig dyncfg 1
    puts [array get dyncfg]
  }

  loadConfigArr conf [loadConfig tivoweb]
  if {$conf(Prefix)!=""} { set conf(Prefix) /[string trim $conf(Prefix) /] }
  if {$conf(Password) != "" || $conf(UserName) != ""} {
    set userpass [base64enc "$conf(UserName):$conf(Password)"]
  } else {
    set userpass ""
  }
}

proc setup_timezone {} {
  global daylightsavings stzoffset tzoffset version

  RetryTransaction {
    if {$version >= 3} {
      set lconfig  [db $::db open /State/LocationConfig]
      if { $version >=4 } {
        set setuptz [dbobj $lconfig get TimeZoneOffset]
      } else {
        set setuptz [dbobj $lconfig get TimeZoneOld]
      }
      if { $version >=5 } {
        # Assumes that UseDaylightSavings uses 0=No, 1=Yes, NA=Yes
        set daylightsavings [dbobj $lconfig get UseDaylightSavings]
        if {$daylightsavings != ""} {incr daylightsavings}
      } else {
        # DaylightSavingsPolicy uses 1=No, 2=Yes
        set daylightsavings [dbobj $lconfig get DaylightSavingsPolicy]
      }
    } else {
      set setup [db $::db open /Setup]
      set setuptz [dbobj $setup get TimeZone]
      set daylightsavings [dbobj $setup get DaylightSavingsPolicy]
    }
  }

  if {$setuptz == ""} { set setuptz 0 }
  if {$daylightsavings == ""} { set daylightsavings 2 }
  if {$::conf(DST_Policy) != 0} {set daylightsavings $::conf(DST_Policy)}

  # Determine Timezone Offset from GMT
  if { $setuptz <= 0 || $version >= 4} {
    set tz $setuptz
  } else {
    set tzlist "-5 -6 -7 -8 -9 -10 0 1 2 3 4 5 6 7 8 9 10 11 12 -1 -2 -3 -4 -11 -1 2"
    set tz [expr [lindex $tzlist [expr $setuptz - 1]]*60*60]
    unset tzlist
  }
  set stzoffset [expr round(([clock scan [lreplace [exec date -u] 4 4]] - [clock seconds] + $tz)/60.0)*60]

  if {$daylightsavings == 2} {
    if {$::uktivo} {
      mySource $::source_dir/libs/tz/GB.itcl
    } else {
      mySource $::source_dir/libs/tz/US.itcl
    }
  } else {
    mySource $::source_dir/libs/tz/GMT.itcl
  }
  # OLD Static DST variable (should use the new: get_tzoffset <time>)
  set tzoffset [expr round(([clock scan [lreplace [exec date -u] 4 4]] - [clock seconds] + [get_tzoffset [clock seconds]])/60.0)*60]
}

proc load_init {pname req {ntries 3}} {
  set count 1
  set attempt ""
  set start [clock seconds]
  while { $count <= $ntries } {
    puts -nonewline "-running $pname$attempt"
    incr count
    set attempt "; attempt $count"
    if [catch {$pname}] {
      puts "--threw error $::errorInfo"
      continue
    }
    set done 1
    foreach var $req {
      if ![info exists ::$var] {
        puts "--failed to create $var"
        set done 0
        break
      }
    }
    if {$done} {
      set count 0
      break
    }
  }
  if { $count } {
    puts "-error running $pname; aborted - $::errorInfo"
  } else {
    puts "--- completed in [expr [clock seconds]-$start] seconds"
  }
}

proc init_channelList {{force 0}} {
  if {![needsReload channelList /Setup] && !$force} return

  if {$force==1} {catch {unset ::channelData}}
  global db

  if [info exists ::channelList] {
    binary scan $::channelList I* tmp
    foreach {bits chansub numbin fsid} $tmp {
      set old($chansub) [list $bits $numbin $fsid]
    }
    unset tmp
  }
  set hdtivo [expr $::dtivo && ([PrefixMatches "3.1.5" $::tivoswversion] || [PrefixMatches "6.3" $::tivoswversion])]
##TODO - save in config, not MFS
##TODO - scan for added/removed channels... only do a full rescan if none are found... use modified?
  RetryTransaction {
    set channelSigtypes {}
    set chanLists {}

    set setup [db $db open /Setup]
    set setupid [dbobj $setup fsid]
    set sources [dbobj $setup get Source]

    foreach source $sources {
      lappend chanLists [dbobj $source gettarget Channel]
      lappend channelSigtypes [dbobj $source get SignalType]
    }
  }
  foreach channels $chanLists sigtype $channelSigtypes {
    set siginfo [expr $sigtype<<28] ;# this puts siginfo in the top 4 bits of chanid
    set channelcount [llength $channels]
    set subpos [string length $setupid/]
    #puts "-Getting basic channel data for $channelcount channels"
    ForeachTrans channel $channels 50 {
      set subid [string range $channel $subpos end]
      set chansubid [expr $subid + $siginfo]
      if [info exists old($chansubid)] {
        set bits [lindex [set olddat $old($chansubid)] 0]
        if {!$force && ($bits&1)==0} {
          set numbin [lindex $olddat 1]
          set num [expr ($numbin<<16)|(($numbin>>16)&0xffff)] ;# change num to sort properly
          lappend chans($num) $bits $chansubid $numbin [lindex $olddat 2]
          continue
        }
        set bits [expr ($bits & ~6)|1] ;# unset the record and favorite bits
      } else { set bits 1 }
      set channelobj [db $db openidconstruction $setupid $subid]
      set numbin [dbobj $channelobj get Number]
      #~ if {[dbobj $channelobj get Record]!=0} {incr bits 2}
      if {[dbobj $channelobj get Record]!=0 || [dbobj $channelobj get Surf]!=0} {incr bits 2}
      set station [dbobj $channelobj gettarget Station]
      #set station [string range $station 0 [expr [string length $station]-4]]
      set station [lindex [split $station /] 0]
      if {[dbobj $channelobj get Favorite]==1} {incr bits 4}
      if {$sigtype==6} {
        if {$numbin>999 || $numbin==90} {set bits 0
        } elseif {$numbin<0} {
          if {!$hdtivo} {set bits 0}
        } elseif {$numbin < 80} {
          # not sure if SA boxes connected to a directv box have sigtype 6, I suspect not --btux9
          if {$::dtivo} {
            set apgobj [dbobj $channelobj get ApgChannel]
            if {[dbobj $apgobj get ChannelPipIndicator]==3} {set bits 0
            } elseif {[dbobj $apgobj get TransmitNetworkId]==7} {set bits 0
            } else {
              set chandefobj [dbobj $apgobj get ChannelDefinition]
              if {[dbobj $chandefobj get NumberOfStreams]<1} {set bits 0}
            }
          }
        }
      }
      set num [expr ($numbin<<16)|(($numbin>>16)&0xffff)] ;# change num to sort properly
      lappend chans($num) $bits $chansubid $numbin $station
    }
  }
  if [info exists old] {unset old}
  set lastfsid 0
  set hi999 [expr 1000<16]
  foreach cnum [lsort -integer [array names chans]] {
    set l $chans($cnum)
    set len [llength $l]
    foreach {bits chansub num fsid} $l {
      if {$len>4 && $num > 99} {
        RetryTransaction {
          set station [db $db openid $fsid]
          set call [dbobj $station get CallSign]
          if {[PrefixMatches "TIVO" $call]} { set bits 0 }
        }
      }
      lappend clist $bits $chansub $num $fsid
    }
  }
  set ::channelList [binary format I* $clist]
  saveToStateMod channelList
}

proc init_sp_cache {} {
  global db
  global cache_sp_moddate
  global cache_sp_fsid
  global cache_sp_key
  global seasonpassdir

  set cache_sp_fsid ""
  set cache_sp_key ""
  set cache_sp_moddate ""

  catch {
    RetryTransaction {
      set spfsid [mfs find $seasonpassdir]
      set spfsid [lindex $spfsid 0]
      set cache_sp_moddate [mfs moddate $spfsid]
    }
  }
  if { $cache_sp_moddate == "" } {
    return
  }

  ForeachMfsFileTrans fsid name "tyDb" $seasonpassdir "" 20 {
    # puts "$fsid $name $type"
    set sp [db $db openid $fsid]
    set type [defaultval 1 [dbobj $sp get Type]]
    if { $type == 1 } {
      set series [dbobj $sp get Series]
      set seriesfsid [dbobj $series fsid]
      set station [dbobj $sp get Station]
      set stationfsid [dbobj $station fsid]
      lappend cache_sp_fsid $fsid
      lappend cache_sp_key "$seriesfsid|$stationfsid"
    }
  }
}

proc init_logoindex {{force 0}} {
  global db logoTable

  #if {![needsReload logoTable /LogoGroup] && !$force} return
  if {$force} {
    unset ::channelData
  } else {
    if {[info exists logoTable]} return
    set tmp [loadFromState logoTable]
    if {$tmp!=""} {
      array set logoTable $tmp
      return
    }
  }

  set LogoGroups {}
  if [catch {
    ForeachMfsFile fsid name type "/LogoGroup" "" {
      set info [split $name :]
      lappend LogoGroups $fsid [lindex $info 0] [lindex $info 2]
    }
  }] return

  RetryTransaction {
    if [info exists logoTable] {unset logoTable}
    foreach {fsid space size} $LogoGroups {
      set LogoGroupEntry [db $db openid $fsid]
      set LogoGroupImages [dbobj $LogoGroupEntry gettarget Image]
      set LogoGroupIndexes [dbobj $LogoGroupEntry get Index]
      set logohi [expr $space<<16]
      foreach targ $LogoGroupImages ndx $LogoGroupIndexes {
        lappend logoTable([expr $ndx | $logohi]) $targ $size
      }
    }
  }
  set idlist [array names logoTable]
  set lastid 0
  #set lastname ""
  for {set x 0} {$x < [llength $idlist]} {incr x 20} {
    RetryTransaction {
      foreach ndx [lrange $idlist $x [expr $x+19]] {
        set ids $logoTable($ndx)
        set sizes 0
        foreach {fsid size} $ids {
          if {$fsid == $lastid} continue
          set sizes [expr $sizes|$size]
          set lastid $fsid
          regexp {([0-9]*)/(.*)} $fsid junk id subid
          set logo [db $db openidconstruction $id $subid]
          set name [dbobj $logo get Name]
          set name [string range $name 0 [expr [string length $name]-7]]
        }
        set logoTable($ndx) $sizes$name
      }
    }
  }
  saveToState logoTable [array get logoTable]
}

proc init_preload_favs {} {
  set index 0
  ForeachChannel chinfo 1 {
    if { ($index&7)==7} update
    incr index
  }
  puts "Preload Favs done"
}

proc base64enc {s} {
  set output {}
  set max [string length $s]
  set pad [expr (99-$max)%3]
  #~ logit 0 max
  append s "\0\0\0"
  set rets ""
  for {set i 0} {$i<$max} {incr i 3} {
    binary scan $s @${i}H6 h
    #~ logit 0 h
    set n 0x$h
    set ll {}
    for {set j 0} {$j<4} {incr j} {
      set ll [concat [expr $n & 0x3f] $ll]
      set n [expr $n >> 6]
    }
    foreach n $ll { append rets [string index "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/" $n] }
  }
  set rets [string range $rets 0 [expr (($max*4+2)/3)-1]]
  append rets [lindex {"" "=" "=="} $pad]
  #~ logit 0 rets
  return $rets
}

proc init_init {} {
  readconfig

  global needsNpkChannel
  set needsNpkChannel [checkSchema "SeasonPass" "NpkChannelDefinition"]

  global StatLink
  if [info exists ::dyncfg(oldVer)] { set v $::dyncfg(oldVer) } else { set v 0 }
  if {$v==$::TWP(DIST_VERSION)} { set StatLink ""
  } else {
    set ::dyncfg(oldVer) $::TWP(DIST_VERSION)
    if $::conf(SF_Stats) {
      set dist $::TWP(DIST_VERSION)
      set hda "00000000"
      if [info exists ::env(HDA_ID)] {set hda $::env(HDA_ID)
      } elseif {[catch { set hda [exec driveid /dev/hda]}] || $hda=="00000000"} {
        set hda 0x7fffffff
        ForeachMfsFileTrans id name type "/Resource" {
          set hda [min [mfs moddate $id] $hda]
        }
      }
      set ver "$dist&sw=$::tivoswversion"
      if [info exists ::tvStandard] {append ver "&tv=$::tvStandard"}
      regsub -all { } $ver {_} ver
      set StatLink "<img src=\"http://tivowebplus.sf.net/cgi-bin/stats.cgi?hda=$hda&ver=$ver\" width=0 height=0>"
    }
  }
# determine current IP (relies on config being loaded)
  global ip_address
  set timer [expr [clock seconds]+45]
  while 1 {
    if { $::conf(EthernetInterface) == "" } {
      set ip_address [lindex [split [string trim [exec ifconfig | grep "inet addr" | grep -v "127.0.0.1"]] " :"] 2]
    } else {
      set ip_address [lindex [split [string trim [exec ifconfig $::conf(EthernetInterface) | grep "inet addr" ]] " :"] 2]
    }
    if { $ip_address != "" && $ip_address != "0.0.0.0" } break
    if {$timer<[clock seconds]} {
      puts "ERROR: Unable to ascertain network IP Address"
      exit 1
    }
    puts "Waiting for network interface"
    after 3000
  }

# Use subnet filtering to more easily handle authentication -- btux9
  global subnet
  if {$::userpass==""} {
    set subnet ""
  } elseif {$::conf(LAN_Auth)} {
    set subnet "x"
  } else {
    regsub {\.[0-9]+$} $ip_address {} subnet
  }
  #~ puts "subnet: $::subnet"

  global hostname
  set hostname $::env(HOSTNAME)
  if {$hostname == "(none)"} {
    set hostname $ip_address
  }

  #~ after cancel cron
  after idle cron
  getGenres
  setup_timezone

  load_init init_channelList {channelList} 1
  load_init init_logoindex logoTable 1
  load_init init_sp_cache {cache_sp_moddate cache_sp_fsid cache_sp_key}

  global g
  set g(groups) {All Received Favorite}

  if {![info exists ::channelData]} {after idle {do_init preload_favs}}
  cronRegister {do_init channelList} 57

  set ret {}
  lappend ret "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"
  lappend ret <HTML> <HEAD>
  switch $::conf(HostInTitle) {
    1     { set titlestr "\$title \[$::hostname\]" }
    2     { set titlestr "\[$::hostname\] \$title" }
    default { set titlestr \$title }
  }
  lappend ret "<TITLE>$titlestr</TITLE>"
  set t ""
  foreach style [list .common \$::dyncfg(style)] {
    if {$style != ".common"} {set t " title='$style'"}
    lappend ret "<link type='text/css' rel='stylesheet' href='$::conf(Prefix)/$style.css'$t>"
  }
  lappend ret \$args </head> <body\$bodyclass> {<div class='header clearfix'>} {<ul id='theme'>}
  lappend ret "<li id=themelink>[html_link / $::TWP(NAME) "Return to Main Menu"]</li>"
  lappend ret "<li id=themeversion title='$::TWP(DIST_VERSION)'><small>v$::TWP(VERSION)</small></li>"
  lappend ret "<li class=hostname>\[$::hostname\]</li>" "</ul>"
  global def_header
  regsub -all {[\{\[\"\\]} [join $ret \n] {\\&} def_header

  # ProgramSource 4=Suggestions, 6=TIVOR
  global PS_4 PS_6
  set PS_4 0
  set PS_6 0
  if $::version3 {
    ForeachMfsFile id name y /SeasonPass/Special {
      set n [string index $name 2]
      set PS_${n} "$id/-1"
    }
  }
}
