## load.tcl - put loading code here so reload will REALLY reload
# added loading support for options to cull based on pay per view, not received
#   and xm channel types.
# Closed socket during reload -- btux9 2006/09/08
# Changed to run url_decode on submitted varnames so arrays can be used --btux9
# temporary patch to daylightsavings to keep 6.2 compatibility --btux9
# change DST patch for something simpler and hopefully more compatible
# implemented load-on-demand for modules -- BTUx9
# Changed so listening socket isn't closed during reloads-- BTUx9
# moved "full reload" code to libs/init.tcl

global last_load hooks

set load_start [clock seconds]

proc do_init args {
  if ![info exists MOD::init_loaded] { namespace eval ::MOD "source $::source_dir/libs/init.tcl" }
  namespace eval ::MOD init_$args
  #~ puts "ran do_init $args"
}

puts "------============------ Loading ------============------"
if !$reload { do_init globals }

set hooks {}
puts "Loading libs..."
set libs [lsort [glob "$source_dir/libs/*.itcl"]]
foreach f $libs {
  set ::errorCode ""
  set ::errorInfo ""
  if {[file mtime $f]>$last_load} {
    puts -nonewline [file rootname [file tail $f]],
    mySource $f ""
  }
}

if !$reload { do_init init }

#puts "Loading utils..."
set utils {}
catch {set utils [lsort [glob "$source_dir/modules/*_util.tcl"]]}
foreach f $utils {
  set ::errorCode ""
  set ::errorInfo ""
  if {[file mtime $f]>$last_load} {
    puts -nonewline [file rootname [file tail $f]],
    mySource $f ""
  }
}
hookem $hooks

global mod_cache
puts -nonewline "\n\nLoading modules..."
if {[loadConfig module.cache 1]==""} {
  set last_load 0
  set changed 1
} else {
  if !$last_load {
    set last_load [file mtime "$::data_dir/config/module.cache"]
    foreach {mod vars} [array get mods] { hookem [lindex $vars 2] }
  }
  set changed 0
}

foreach f [lsort [glob "$source_dir/modules/*.itcl"]] {
  set mod [file rootname [file tail $f]]
  if {[info exists mods($mod)] && [file mtime $f]<$last_load} continue
  set ::errorCode ""
  set ::errorInfo ""
  clearModules
  set changed 1
  set hooks {}
  if [set ret [catch {namespace eval MOD "source $f"}]] {
    if {$ret!=4} { print_html_error "" "Error loading module $mod" $::errorInfo }
  }
##xtodo - unset a mod's hooks if reloading
  if [info exists mods($mod)] {
    foreach {s a h} $mods($mod) break
    unhookem $h
  }
  set acts {}
  set size 0
  set incompat 1
  if {[info exists MOD::compat]} {
    foreach c $MOD::compat {
      if [PrefixMatches $c $::TWP(VERSION)] {
        set incompat 0
        break
      }
    }
  }
  if {$incompat} { lappend linc $mod
  } else {
    foreach p [info commands MOD::action_*] { lappend acts [string range $p 14 end]}
    if {$acts=={} && [info commands MOD::*]=={}} { lappend lfeat $mod
    } else {
      puts -nonewline "$mod,"
      set size 1
      hookem $hooks
    }
  }
  set mods($mod) [list $size $acts $hooks]
}
clearModules
if [info exists linc] {
  puts "\n--Warning: The following modules are incompatible with this version of TWP"
  foreach m $linc { puts -nonewline $m, }
}
if [info exists lfeat] {
  puts "\n--Info: The following modules won't be loaded, they may be disabled or rely on an unavailable feature (e.g. events or sendkey)"
  foreach m $lfeat { puts -nonewline $m, }
}
puts "\n"
if {$changed} {
  set mod_cache ","
  foreach f [array names mods] {
    set mod [file rootname [file tail $f]]
    set vars $mods($mod)
    foreach {size acts hooks} $vars break
    if {!$size} continue
    append mod_cache [join $acts ","] ",$mod:$size,"
  }
  set outs "array set mods \{[array get mods]\}\nset mod_cache \"$mod_cache\"\n"
  #~ foreach x {mod_cache linc} {
  #~ }
  saveConfig module.cache $outs
}
make_menu

unset reload
set last_load $load_start
puts "Loading completed in [expr [clock seconds] - $load_start] seconds"
