# channelgrid sourced to lower memory footprint

  # Start a timer so that we will know how long it took to render the page
  set startTimer [clock seconds]

  puts $chan [html_start "Channel Grid"]

  # Column start values

  set startColumn     [expr $HOUR_START * 3600]
  set startTime       [expr $startColumn]
  set endTime         [expr $startTime + 86400]

  # Monday signifies the "start" of the week, but is
  # configurable so may not actually be on a Monday :)

  if {$startWeek != ""} {
    set thisMonday      $startWeek
  } else {
    set currentTimeDate [expr [clock scan "12:00 AM"] - $tzoffset]
    set dateNum         [clock format $currentTimeDate -format "%u"]

    # Make offset equal to the number of days since the last START_DAY.
    # "thisMonday" is now used to identify the first day of the grid.
    set offset [clock format [expr [clock scan "12:00 AM $START_DAY"] - $tzoffset] -format "%u"]
    set offset [expr $dateNum - $offset]
    if {$offset < 0} {set offset [expr $offset + 7]}

    set thisMonday [expr $currentTimeDate - (($offset) * 86400)]
  }

  set prevMonday     [clock scan "-7 day" -base $thisMonday]
  set nextMonday     [clock scan  "7 day" -base $thisMonday]

  #
  # Get the Schedule
  #

  set thisMondayDate  [expr $thisMonday/86400]
  set nextMondayDate  [expr $nextMonday/86400]

  if { $whichChannel != 0 } {
    getStationData chinfo $whichChannel
    #set num  [lindex $data 0]
    #set sign [lindex $data 2]
    #set name [lindex $data 3]
    #set asciiChannel "$num - $sign <SMALL>($name)</SMALL>"
    set asciiChannel "$chinfo(stanum) - $chinfo(sign) <SMALL>([strim $chinfo(name)])</SMALL>"

    puts $chan "<SMALL>[bodytext_start]"
    puts $chan "Getting Schedule Data..."
    set scheduleList {}
    ForeachMfsFileTrans fsid name type "/Schedule" "$whichChannel:" 15 {
      if ![catch {set stationday [db $db openid $fsid]}] {
        set showings [dbobj $stationday get Showing]
        foreach showing $showings {
          if ![catch {set date [dbobj $showing get Date]}] {
            if { $date >= $thisMondayDate && $date <= $nextMondayDate } {
              set showingfsid "[dbobj $showing fsid]/[dbobj $showing subobjid]"
              set programfsid [dbobj $showing gettarget Program]
              set duration    [dbobj $showing get Duration]
              set time        [dbobj $showing get Time]

              # Round time to closest 15 minutes
              set time [expr (($time+$GRiD)/$GRID)*$GRID]

              # Refine our query to deal with timezones
              set datetime [expr $date * 86400 + $time]
              if { $datetime >= $thisMonday && $datetime < $nextMonday } {
                # Round Duration to closest 15 minutes (minimum 15 minutes)
                if { $duration > $GRiD } {
                  set duration [expr (($duration+$GRiD)/$GRID)*$GRID]
                } else {
                  set duration $GRID
                }
                lappend scheduleList [list $showingfsid $programfsid $time $date $duration]
              }
            }
          }
        }
        unset showings
      } else {
        # Signal that an error was found in the EPG
        puts -nonewline $chan "<font color=red>!</font>"
      }
    }
    puts $chan "Done.<BR>"

    set entries [llength $scheduleList]
    puts $chan "Getting Program Data <SMALL>(for $entries entries)</SMALL>..."
    set programList {}
    set entriesPerRound 50
    set rounds [expr $entries / $entriesPerRound]

    for { set x 0 } { $x <= $rounds } { incr x } {
      set y [ expr $x * $entriesPerRound ]
      set last [ expr ( $x + 1 ) * $entriesPerRound ]
      if { $last > $entries } {
        set last $entries
      }

      RetryTransaction {
        for {  } { $y < $last } { incr y } {

          set sl [lindex $scheduleList $y]

          # Attempt to free up some memory on the fly?
          set scheduleList [lreplace $scheduleList $y $y {}]

          set constprogramfsid [lindex $sl 1]
          set programfsid [lindex [split $constprogramfsid {/}] 0]

          if ![catch {set program [db $db openid $programfsid]}] {

            set seriesfsid ""
            catch {set seriesfsid     [dbobj $program gettarget Series]}
            set title ""
            catch {set title [strim   [dbobj $program get Title]]}
            set eptitle ""
            catch {set eptitle [strim [dbobj $program get EpisodeTitle]]}

            if {$SHOW_DESCRIPTION != "false"} {
              set description ""
              catch {set description [strim [dbobj $program get Description]]}

              set origairdate ""
              catch {set origairdate [dbobj $program get OriginalAirDate]}
              if { $origairdate != "" } {
                set origairdatesecs [expr $origairdate * 86400 - $tzoffset]
                if {$::lang == "en"} {
                  set daystr [nth [clock format $origairdatesecs -format "%e"]]
                  set airdatestr [clock format $origairdatesecs -format "%a $daystr %b %Y"]
                } else {
                  set airdatestr [clock format $origairdatesecs -format "%1m/%1d/%Y"]
                }
              } else {
                set airdatestr ""
              }

              set description "$airdatestr $description"
            } else {
              set description ""
            }

            lappend sl $title $eptitle $description $seriesfsid
            lappend programList $sl
          } else {
            # Signal that an error was found in the EPG
            puts -nonewline $chan "<font color=red>!</font>"
          }
        }
      }
    }
    catch {unset sl}
    unset scheduleList
    puts $chan "Done.<BR>"

    set legend 0
    puts $chan "Building Grid Entries..."
    foreach program $programList {
      set showingfsid     [lindex $program 0]

      set time            [lindex $program 2]
      set date            [lindex $program 3]
      set duration        [lindex $program 4]
      set title           [lindex $program 5]
      set eptitle         [lindex $program 6]
      set description     [lindex $program 7]
      set constseriesfsid [lindex $program 8]

      set seriesfsid [lindex [split $constseriesfsid {/}] 0]

      set programKey  [expr $date * 86400 + $time]

      # Check to see if there is already an entry for this timeslot
      if ![catch {set programList $foo($programKey)}] {
        set getShowingfsid [lindex $programList 4]
        if { $getShowingfsid != {} } {
          # Mark the old entry with a * to signal collision, as this
          # recording is in the same slot, and link to this recording
          set getShowName [html_encode $title]
          regsub -all {\"} $getShowName {\&quot;} getShowName
          #~ set attrs "title=\"$getShowName\""

          set title       [lindex $programList 0]
          set title "$title[html_link "/showing/$showingfsid" "*" $getShowName {STYLE="text-decoration:none;color:red"}]"
          set eptitle     [lindex $programList 1]
          set description [lindex $programList 2]
          set seriesfsid  [lindex $programList 3]
          set showingfsid [lindex $programList 4]
          set duration    $GRID
          set legend      1
        }
      }

      set programList {}
      lappend programList $title $eptitle $description $seriesfsid $showingfsid
      set foo($programKey) $programList

      # Find the number of 15 minute slots
      set slots [expr ($duration+$GRiD)/$GRID]

      for {set slotCount 1} {$slotCount<$slots} {incr slotCount} {
        set programKey [expr $programKey + $GRID]
        set programList {}
        lappend programList {SKIP}
        set foo($programKey) $programList
      }
    }
    puts $chan "Done.<BR>"

    if {$MARK_RECORDINGS != 0} {
      puts $chan "Mark Recordings..."
      ForeachMfsFileTrans fsid name type "/Recording/Active" "4" 15 {
        set rec         [db $db openid $fsid]
        set showing     [dbobj $rec     get    Showing]
        set date        [dbobj $showing get    Date]
        set station     [dbobj $showing get    Station]
        set stationfsid [dbobj $station fsid]

        if {$::version >= 3} {
          set recbeh  [dbobj $rec    get RecordingBehavior]
          set presbeh [dbobj $recbeh get PresentationBehavior]
        } else {
          set presbeh [dbobj $rec get SelectionType]
        }

        if { $date >= $thisMondayDate && $date <= $nextMondayDate } {
          set time     [dbobj $showing get Time]
          set duration [dbobj $showing get Duration]

          # Round time to closest 15 minutes
          set time [expr (($time+$GRiD)/$GRID)*$GRID]

          # Refine our query to deal with timezones
          set datetime [expr $date * 86400 + $time]
          if { $datetime >= $thisMonday && $datetime < $nextMonday } {
            # Find the number of 15 minute slots
            set slots        [expr ($duration+$GRiD)/$GRID]
            if {$slots == 0} {set slots 1}
            set programKey   [expr $date * 86400 + $time]

            if {$whichChannel == $stationfsid} {
              if {$presbeh != 6} {
                # Mark the timeslot as having a recording on this channel(1).
                # Will be displayed with a red outline.
                for {set slotCount 1} {$slotCount<=$slots} {incr slotCount} {
                  set bar($programKey) 1
                  set programKey [expr $programKey + $GRID]
                }
                set legend [expr $legend | 2]
              } else {
                # Mark the timeslot as having a suggested recording on this
                # channel(2).  Will be displayed with a dotted orange outline.
                for {set slotCount 1} {$slotCount<=$slots} {incr slotCount} {
                  set bar($programKey) 2
                  set programKey [expr $programKey + $GRID]
                }
                set legend [expr $legend | 4]
              }
            } else {
              if {$MARK_RECORDINGS > 1} {
                # Mark the timeslot as having a recording on a different
                # channel(3) or a suggested recording on a differnet
                # channel(4), but only if it has not already been marked as
                # having a recording (on this channel).  Will be displayed
                # with a dashed or dotted yellow outline.
                for {set slotCount 1} {$slotCount<=$slots} {incr slotCount} {
                  set marked 0
                  catch {set marked $bar($programKey)}
                  if {$marked != 1 && $marked != 2} {
                    if {$marked == 3 && $MARK_RECORDINGS > 2} {
                      # There is already a recording on another channel
                      # so this must be a dual-tuner unit, mark it as
                      # such(5).  Will be displayed with a solid yellow
                      # outline.
                      set bar($programKey) 5
                      set legend [expr $legend | 32]
                    } else {
                      if {$presbeh != 6} {
                        set bar($programKey) 3
                        set legend [expr $legend | 8]
                      } else {
                        set bar($programKey) 4
                        set legend [expr $legend | 16]
                      }
                    }
                  }
                  set programKey [expr $programKey + $GRID]
                }
              }
            }
          }
        }
      }
      puts $chan "Done.<BR>"
    }
    puts $chan "[bodytext_end]</SMALL>"

    # If we have marked any shows/timeslots then display an appropriate
    # legend with information on the markings.
    if {$legend != 0} {
      puts $chan "<FONT SIZE=1><BR></FONT><TABLE>"
      if {[expr $legend & 1]} {
        puts $chan "<TR STYLE=font-size:11px><TD><FONT COLOR=red>*</FONT></TD><TD>identifies additional shows in this timeslot</TD></TR>"
      } else {
        puts $chan "<TR STYLE=font-size:0px><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>"
      }
      if {[expr $legend & 2]} {
        puts $chan "<TR STYLE=font-size:11px><TD STYLE=\"border-color:red;border-width:1px;border-style:dashed\">&nbsp;</TD><TD>identifies a recording within this timeslot</TD></TR>"
      }
      if {[expr $legend & 4]} {
        puts $chan "<TR STYLE=font-size:11px><TD STYLE=\"border-color:orange;border-width:1px;border-style:dotted\">&nbsp;</TD><TD>identifies a suggested recording within this timeslot</TD></TR>"
      }
      if {[expr $legend & 8]} {
        puts $chan "<TR STYLE=font-size:11px><TD STYLE=\"border-color:yellow;border-width:1px;border-style:dashed\">&nbsp;</TD><TD>marks a recording (on another channel) within this timeslot</TD></TR>"
      }
      if {[expr $legend & 16]} {
        puts $chan "<TR STYLE=font-size:11px><TD STYLE=\"border-color:yellow;border-width:1px;border-style:dotted\">&nbsp;</TD><TD>marks a suggested recording (on another channel) within this timeslot</TD></TR>"
      }
      if {[expr $legend & 32]} {
        puts $chan "<TR STYLE=font-size:11px><TD STYLE=\"border-color:yellow;border-width:1px;border-style:solid\">&nbsp;</TD><TD>marks two recording (on other channels) within this timeslot</TD></TR>"
      } else {
        puts $chan "<TR STYLE=font-size:0px><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>"
      }
      puts $chan "</TABLE>"
    }
  } else {
    set asciiChannel "ToDo Schedule"

    #
    # Display a schedule of the shows to be recorded this week
    #
    puts $chan "<SMALL>"
    puts $chan "Building Schedule..."
    set legend 0

    ForeachMfsFileTrans fsid name type "/Recording/Active" "4" 15 {
      set rec      [db $db openid $fsid]
      set manual   0

      if {$::version3} {
        set recbeh     [dbobj $rec    get RecordingBehavior]
        set presbeh    [dbobj $recbeh get PresentationBehavior]
        set progbeh    [dbobj $recbeh get ProgramGuideBehavior]

        if { $presbeh == 10 || $presbeh == 9 } { set manual 1 }
        if { $progbeh == 3  || $progbeh == 8 } { set manual 1 }
      } else {
        set seltype [dbobj $rec get SelectionType]
        if { $seltype == 10 || $seltype == 5 } { set manual 1 }
      }

      set showing      [dbobj $rec     get       Showing]
      set showingfsid  [dbobj $rec     gettarget Showing]
      set station      [dbobj $showing get       Station]
      set stationfsid  [dbobj $station fsid]
      set program      [dbobj $showing get       Program]
      set title [strim [dbobj $program get       Title]]
      set date         [dbobj $showing get       Date]
      set time         [dbobj $showing get       Time]
      set duration     [dbobj $showing get       Duration]

      if {$SHOW_DESCRIPTION != "false"} {
        set description [strim [dbobj   $program  get Description]]
      } else {
        set description ""
      }

      if { $manual } {
        if { $title == "" } {
          set title "Manual Recording"
        }
        set series ""
        set seriesfsid $showingfsid
        set episodic 1
      } else {
        set series [dbobj $program get Series]
        set seriesfsid [dbobj $series fsid]
        set episodic [defaultval 1 [dbobj $series get Episodic]]
      }

      if { $episodic == 1 } {
        set eptitle [strim [dbobj $program get EpisodeTitle]]
      } else {
        set eptitle ""
      }

      # Round time to closest 15 minutes
      set time [expr (($time+$GRiD)/$GRID)*$GRID]

      set programKey  [expr $date * 86400 + $time]

      # Check to see if there is already an entry
      if ![catch {set programList $foo($programKey)}] {
        set getShowingfsid [lindex $programList 4]
        if { $getShowingfsid == {} } {
          # Mark the new entry with a + to signal overlap, as this
          # recording is starting before the other has completed.
          set title "$title<FONT COLOR=red><SMALL>+</SMALL></FONT>"
          set legend [expr $legend | 1]
        } else {
          # Mark the old entry with a * to signal collision, as this
          # recording is starting at the same time, and link to this
          # recording
          set getShowName [html_encode $title]
          regsub -all {\"} $getShowName {\&quot;} getShowName
          #~ set attrs "title=\"$getShowName\""

          set title       [lindex $programList 0]
          set title "$title[html_link "/showing/$showingfsid" "*"  $getShowName {STYLE="text-decoration:none;color:red"}]"
          set eptitle     [lindex $programList 1]
          set description [lindex $programList 2]
          set seriesfsid  [lindex $programList 3]
          set showingfsid [lindex $programList 4]
          set duration    $GRID
          set legend      [expr $legend | 2]
        }
      }

      set programList {}
      lappend programList $title $eptitle $description $seriesfsid $showingfsid
      set foo($programKey) $programList

      # Find the number of 15 minute slots
      set slots [expr ($duration+$GRiD)/$GRID]

      for {set slotCount 1} {$slotCount<$slots} {incr slotCount} {
        set programKey [expr $programKey + $GRID]
        set programList {}
        lappend programList {SKIP}
        set foo($programKey) $programList
      }
    }
    puts $chan "Done.<BR>"

    if {$MARK_RECORDINGS != 0} {
      puts $chan "Mark Recordings..."
      ForeachMfsFileTrans fsid name type "/Recording/Active" "4" 15 {
        set rec         [db $db openid $fsid]
        set showing     [dbobj $rec     get    Showing]
        set date        [dbobj $showing get    Date]

        if {$::version >= 3} {
          set recbeh  [dbobj $rec    get RecordingBehavior]
          set presbeh [dbobj $recbeh get PresentationBehavior]
        } else {
          set presbeh [dbobj $rec get SelectionType]
        }

        if { $date >= $thisMondayDate && $date <= $nextMondayDate } {
          set time     [dbobj $showing get Time]
          set duration [dbobj $showing get Duration]

          # Round time to closest 15 minutes
          set time [expr (($time+$GRiD)/$GRID)*$GRID]

          # Refine our query to deal with timezones
          set datetime [expr $date * 86400 + $time]
          if { $datetime >= $thisMonday && $datetime < $nextMonday } {
            # Find the number of 15 minute slots
            set slots        [expr ($duration+$GRiD)/$GRID]
            if {$slots == 0} {set slots 1}

            set programKey   [expr $date * 86400 + $time]

            if {$MARK_RECORDINGS > 1} {
              # Mark the timeslot as having a suggested recording on
              # a differnet channel(4).  Will be displayed with a
              # dashed or dotted yellow outline.
              for {set slotCount 1} {$slotCount<=$slots} {incr slotCount} {
                if {$presbeh == 6} {
                  set bar($programKey) 4
                  set legend [expr $legend | 16]
                }
                set programKey [expr $programKey + $GRID]
              }
            }
          }
        }
      }
      puts $chan "Done.<BR>"
    }

    puts $chan "</SMALL>"

    if {$legend != 0} {
      puts $chan "<FONT SIZE=1><BR></FONT><TABLE>"
      if {[expr $legend & 2]} {
        puts $chan "<TR STYLE=font-size:11px><TD><FONT COLOR=red>*</FONT></TD><TD>identifies more than one recording in this timeslot</TD></TR>"
      }
      if {[expr $legend & 1]} {
        puts $chan "<TR STYLE=font-size:11px><TD><FONT COLOR=red><SMALL>+</SMALL></FONT></TD><TD>identifies this recording overlaps the previous one</TD></TR>"
      }
      puts $chan "</TABLE>"
    }

    # Turn this off... it is not required and will speed up the creation of the grid
    #set MARK_RECORDINGS 0
  }

  set startDate     $thisMonday
  set endDate       [expr $thisMonday + (86400 * 6)]

  set startDateDesc [clock format [expr $startDate  + $tzoffset] -format "%B %d, %Y"]
  set endDateDesc   [clock format [expr $endDate    + $tzoffset] -format "%B %d, %Y"]
  set week          [clock format [expr $thisMonday + $tzoffset] -format "%V"]

  #
  # Start rendering the results
  #

  puts $chan "<SMALL><BR></SMALL>"
  puts $chan "<TABLE BORDER=1 CELLPADDING=3 CELLSPACING=1 WIDTH=\"95%\" ALIGN=\"CENTER\">"
  puts $chan "<TR VALIGN=\"TOP\">"
  puts $chan "  <TH ALIGN=\"CENTER\" COLSPAN=\"8\" STYLE=\"font-size:16pt\"><B>$asciiChannel</B></TH>"
  puts $chan "</TR>"
  puts $chan "<TR VALIGN=\"TOP\">"
  puts $chan "  <TD ALIGN=\"CENTER\" COLSPAN=\"8\"><B>[html_link "/channelgrid?startWeek=$prevMonday&whichChannel=$whichChannel" "<<<"] &nbsp; Week $week: $startDateDesc - $endDateDesc &nbsp; [html_link "/channelgrid?startWeek=$nextMonday&whichChannel=$whichChannel" ">>>"]</B></TD>"
  puts $chan "</TR>"

  puts $chan "<TR VALIGN=\"TOP\">"
  puts $chan "  <TD WIDTH=\"9%\">&nbsp</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "0 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "1 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "2 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "3 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "4 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "5 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:11pt\">[clock format [expr [clock scan "6 day" -base [expr $thisMonday + $tzoffset]]] -format "%A<SMALL><BR>%d %b</SMALL>"]</TD>"
  puts $chan "</TR>"

  set col(1) 0
  set col(2) 0
  set col(3) 0
  set col(4) 0
  set col(5) 0
  set col(6) 0
  set col(7) 0

  set totalColumns 0

  for {set x $startTime} {$x < $endTime} {incr x $GRID} {
    puts $chan {<TR VALIGN="TOP">}

    set columnStamp   [clock format $startColumn -format "%l:%M<SMALL>%p</SMALL>"]
    set columnCount   0
    set rowsRemaining [expr ($endTime - $x) / $GRID]

    puts $chan "  <TD WIDTH=\"68\" NOWRAP STYLE=\"font-size:10pt\">$columnStamp</TD>"

    for {set y $startDate} {$y <= $endDate} {incr y 86400} {
      set index       [expr $x + $y]
      set columnCount [expr $columnCount + 1]

      set getValue ""

      if ![catch {set getValue $foo($index)}] {
        set getShowName    [lindex $getValue 0]

        if { $getShowName != {SKIP} } {
          set getEpTitle     [lindex $getValue 1]
          set getDescription [lindex $getValue 2]
          set getSeriesfsid  [lindex $getValue 3]
          set getShowingfsid [lindex $getValue 4]

          set col($columnCount) 1

          set rowSpan 1
          set tempindex $index

          set marked  0
          if ${MARK_RECORDINGS} {catch {set marked $bar($tempindex)}}

          set done 0
          while { $done == 0 } {
            set tempindex [expr $tempindex + $GRID]
            if ![catch {set getVal $foo($tempindex)}] {
              set name [lindex $getVal 0]
              if { $name != {SKIP} } {
                set done 1
              } else {
                set rowSpan [expr $rowSpan + 1]
                if {$MARK_RECORDINGS && $marked != 1 && $marked != 2} {
                  # We want a recording on this channel (marked=1/2), to
                  # override a recording on a different one (marked=3/4/5)
                  catch {set marked $bar($tempindex)}
                }
              }
            } else {
              set done 1
            }
          }

          if {$rowSpan > $rowsRemaining} {set rowSpan $rowsRemaining}

          if {$marked == 0} {
            puts $chan "<TD ROWSPAN=\"$rowSpan\">"
          } else {
            if {$marked == 1} {
              puts $chan "<TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:red;border-width:1px;border-style:dashed\">"
            } else {
              if {$marked == 2} {
                puts $chan "<TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:orange;border-width:1px;border-style:dotted\">"
              } else {
                if {$marked == 3} {
                  puts $chan "<TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:dashed\">"
                } else {
                  if {$marked == 4} {
                    puts $chan "<TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:dotted\">"
                  } else {
                    puts $chan "<TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:solid\">"
                  }
                }
              }
            }
          }

          if {$SHOW_DESCRIPTION == "hover"} {
            set getDescription [html_encode $getDescription]
            regsub -all {\"} $getDescription {\&quot;} getDescription
            #~ set attrs "title=\"$getDescription\""
            set getDescription ""
          } else {
            #~ set attrs ""
          }

          puts $chan "  <B>[html_link "/showing/$getShowingfsid" $getShowName "" {STYLE="font-size:11pt"}]</B>"
          if {$getEpTitle != ""} {
            puts $chan "  <BR><SMALL>[html_link "/showing/$getShowingfsid" $getEpTitle "" {STYLE="font-size:10pt"}]</SMALL>"
          }

          if {$getDescription != ""} {
            puts $chan "  <BR><SMALL><I>[html_link "/showing/$getShowingfsid" $getDescription "" {STYLE="font-size:9pt"}]</I></SMALL>"
          }
          puts $chan "</TD>"
        } else {
          if { $col($columnCount) == 0 } {
            # These are cells at the start of the day marked as SKIP.
            # Flag the cell as empty will allow us to check it for
            # any recordings that started the previous day.
            set getValue ""
          }
        }
      }

      if {$getValue == ""} {
        if {$MARK_RECORDINGS} {
          if {[catch {set marked $bar($index)}] != 0} {
            # Cell is empty and no recordings are schedule within it
            puts $chan "  <TD>&nbsp;</TD>"
          } else {
            # Must be a recording that either started the previous day
            # (eg we are at the start of a column and getShowName=SKIP),
            # or a manual recording that occurs during a slot with no
            # EPG information.
            # Need to find how many rows to span... these are the ones
            # marked to be recorded, but have either no information or
            # are marked with SKIP.
            if {$marked > 0} {
              set rowSpan 1
              set tempindex $index
              set done 0
              while { $done == 0 } {
                set tempindex [expr $tempindex + $GRID]
                set getValue {SKIP}
                catch {set getValue $foo($tempindex)}
                set getShowName [lindex $getValue 0]
                # removed $ from "info exists $bar"... assumed wrong  --btux9
                if {[info exists bar($tempindex)] && $getShowName == {SKIP}} {
                  # Check we have not gone onto the next day, and that the "type"
                  # of recording is the same as that we started with (ie 1/2/3/4/5).
                  if {$rowSpan < $rowsRemaining && $bar($tempindex) == $marked} {
                    set rowSpan [expr $rowSpan + 1]
                    set bar($tempindex) 0
                  }
                } else {
                  set done 1
                }
              }

              if {$marked == 1} {
                puts $chan "  <TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:red;border-width:1px;border-style:dashed\">&nbsp;</TD>"
              } else {
                if {$marked == 2} {
                  puts $chan "  <TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:orange;border-width:1px;border-style:dotted\">&nbsp;</TD>"
                } else {
                  if {$marked == 3} {
                    puts $chan "  <TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:dashed\">&nbsp;</TD>"
                  } else {
                    if {$marked == 4} {
                      puts $chan "  <TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:dotted\">&nbsp;</TD>"
                    } else {
                      puts $chan "  <TD ROWSPAN=\"$rowSpan\" STYLE=\"border-color:yellow;border-width:1px;border-style:solid\">&nbsp;</TD>"
                    }
                  }
                }
              }
            }
          }
        } else {
          puts $chan "  <TD>&nbsp;</TD>"
        }
      }
      set totalColumns [expr $totalColumns + 1]
    }

    puts $chan {</TR>}

    # update time stamp for row
    set startColumn [expr $startColumn + $GRID]
  }

  puts $chan "<TR>"
  puts $chan "  <TD WIDTH=\"9%\">&nbsp</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "0 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "1 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "2 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "3 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "4 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "5 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "  <TD WIDTH=\"13%\" STYLE=\"font-size:10pt\">[clock format [expr [clock scan "6 day" -base [expr $thisMonday + $tzoffset]]] -format "%A"]</TD>"
  puts $chan "</TR>"

  puts $chan {</TABLE>}

  if { $whichChannel != 0 } {
    puts $chan "<SMALL>[bodytext_start][html_link "/channelgrid?whichChannel=0" "View a grid of the Recording Schedule"][bodytext_end]<BR></SMALL>"
  }

  puts $chan "<FONT STYLE=font-size:9px>[bodytext_start]Page took "
  puts $chan [expr [clock seconds] - $startTimer]
  puts $chan "seconds to render[bodytext_end]</FONT><br>"

  catch {unset foo}
  catch {unset bar}
