#!/tvbin/tivosh
#
# Copyright (C) 2003 - Brandon Hill
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; including all implied warranties.
#
# For more information, see the complete GPL at: www.gnu.org
#
# Revision History:
#
# Date     Vers Coments
# -------- ---- --------------------------------------------------
# 01/12/04 V1.0 Initial public release
# 01/13/04      Added v2.5.5 support
# 01/16/04 V1.1 Integrated all Version support, misc minor updates

# OSD console fifo
set OSD             "/tmp/osdconsole"

set OutFile         "/tmp/autospace.log"
set BackGroundFlag  "BG"

set Reload_Interval 1800
set TivoCentral     2
set nowShowing      7

set Prog            "AutoSpace"
set Vers            "V1.1"
set Priority        "fifo 1"

# Initialize Variables for Callback
set EventWait       0
set Restart         0
set Exit            0

set CurrentContext  0
set PreviousContext 0
set DataAge         0
set cons            0

#   Background                  Foreground
set Color(0) "\033\13341m"; set Color(6)  "\033\13331m"; # Overhead
set Color(1) "\033\13342m"; set Color(7)  "\033\13332m"; # Permanent
set Color(2) "\033\13343m"; set Color(8)  "\033\13333m"; # Recordings
set Color(3) "\033\13349m"; set Color(9)  "\033\13339m"; # Suggestions
set Color(4) "\033\13348m"; set Color(10) "\033\13338m"; # Deleted
set Color(5) "\033\13348m"; set Color(11) "\033\13338m"; # Free Space

# Set context init string
set    init "\033\133+p";        # Reset to initial state
append init "\033\1331;5;05;0y"; # Change Black
append init "\033\1331;5;11;1y"; # Change Red
append init "\033\1331;5;10;2y"; # Change Green
append init "\033\1331;5;12;3y"; # Change Yellow
append init "\033\1331;5;15;4y"; # Change Blue
append init "\033\1331;5;01;5y"; # Change Magenta
append init "\033\1331;5;10;6y"; # Change Cyan
append init "\033\1331;5;04;7y"; # Change White
append init "\033\1331;5;00;8y"; # Change Transparent
append init "\033\1331;5;14;9y"; # Change Charcoal Transparent
append init "\033\1331;2;7;46y"; # Change offsets
append init "\033\1331;4;2y";    # Change number of usable rows
append init "\033\1331;1y";      # Clear Page Buffer
append init "\033\13337;48m";    # Default colors

#
# Additional Event namespaces
#

namespace eval TmkEventII {
 variable EVT_USREVENT  98
}

namespace eval TmkUsrEvent {
 variable USR_PING    94
 variable USR_ACK     95
 variable USR_NACK    96
 variable USR_RESTART 97
 variable USR_EXIT    98
}

proc max {a b} {return [expr $a>$b?$a:$b]}

#
# Background error handler (print error and exit)
#

proc bgerror { error } {
 global errorInfo errorCode argv0 cons
 puts stderr "[clock format [clock seconds] -format "%D %T"] $argv0 FATAL: bgerror: $error"
 puts stderr "$errorInfo"
 puts stderr "$errorCode"
 if { $cons != 0 } {
  puts -nonewline $cons "\033\1331;1y\033\133+p"
  close $cons
 }
 exit 1
}

#
# Process events
#

proc EventCallback { type subtype } {
 global EventData
 switch -- $type \
  $TmkEvent::EVT_MW_STATUS {
   if { $subtype == $TmkEventMwStatus::CONTEXT_CHANGE } {
    global PreviousContext CurrentContext EventWait
    set PreviousContext $CurrentContext
    binary scan $EventData I CurrentContext
    if { $EventWait == -1 } { set EventWait $TmkEvent::EVT_MW_STATUS }
   }
  } \
  $TmkEventII::EVT_USREVENT {
   switch -- $subtype \
    $TmkUsrEvent::USR_RESTART {
     global Exit Restart EventWait
     set Exit 1
     set Restart 1
     if { $EventWait == -1 } { set EventWait $TmkEventII::EVT_USREVENT }
    } \
    $TmkUsrEvent::USR_EXIT {
     global Exit EventWait
     if { $Exit } {
      global cons
      if { $cons != 0 } {
       puts -nonewline $cons "\033\1331;1y\033\133+p"
       close $cons
      }
      exit 1
     }
     set Exit 1
     if { $EventWait == -1 } { set EventWait $TmkEventII::EVT_USREVENT }
    } \
    $TmkUsrEvent::USR_PING {
     event send $TmkEventII::EVT_USREVENT $TmkUsrEvent::USR_ACK .
    } \
    $TmkUsrEvent::USR_ACK  {
     global PingAck
     set PingAck 1
    } \
    $TmkUsrEvent::USR_NACK {
     global PingAck
     set PingAck 0
    }
  }
}

#
# Start of main section
#

# Condense argument lists to a single string
set argv [join $argv]
set argc [llength $argv]

if { [lindex $argv [expr $argc-1]] != "$BackGroundFlag" } {

#
# Running in foreground
#

 if { [lindex $argv 0] != "start" } {
  # Register for internal events
  event register $TmkEventII::EVT_USREVENT  EventCallback

  # Check for existing daemon (Send USR_PING, check for USR_ACK / USR_NACK)
  event send $TmkEventII::EVT_USREVENT $TmkUsrEvent::USR_PING .
  set Timer [after 1000 EventCallback $TmkEventII::EVT_USREVENT $TmkUsrEvent::USR_NACK]
  vwait PingAck
  if { $Timer != 0 } { after cancel $Timer; set Timer 0 }
  set Status $PingAck
 } else {
  set Status 0
 }

 switch [lindex $argv 0] \
  "" - "start" {
   # If not already running, restart in background with background flag
   if { $Status } {
    puts stderr "FATAL: Cannot start, daemon is already active"
   } else {
    # Check for osdcons fifo
    if { !([file exists $OSD] && \
           [file type $OSD] == "fifo" && \
           ![catch { set cons [open $OSD {WRONLY NONBLOCK}] } ] ) } {
     if { [catch {set pid [exec osdcons >&/dev/null &]}] && \
          [catch {set pid [exec [file dirname [info script]]/osdcons >&/dev/null &]}] } {
      puts stderr "FATAL: osdcons not active and cannot be started ($OSD not a writable fifo)"
      exit 1
     } else {
      after 1000
      if { [catch {exec setpri $Priority $pid}] && \
           [catch {exec [file dirname [info script]]/setpri $Priority $pid}] } {
       set Priority "default"
      }
      if { !([file exists $OSD] && \
             [file type $OSD] == "fifo" && \
             ![catch { set cons [open $OSD {WRONLY NONBLOCK}] } ] ) } {
       puts stderr "FATAL: osdcons not active and cannot be started ($OSD not a writable fifo)"
       exit 1
      } else {
       puts stderr "INFO: osdcons started (Pid: $pid, Priority: $Priority)"
      }
     }
    }
    puts -nonewline $cons "\033\1331;1y\033\133+p"
    close $cons
    exec $argv0 $BackGroundFlag >&$OutFile &
    puts stderr "INFO: daemon started, output is in $OutFile"
   }
  } \
  "status" {
   if { $Status } {
    puts stderr "INFO: daemon is active"
   } else {
    puts stderr "INFO: daemon is not active"
   }
  } \
  "restart" {
   if { $Status } {
    event send $TmkEventII::EVT_USREVENT $TmkUsrEvent::USR_RESTART .
    puts stderr "INFO: restart event sent"
   } else {
    puts stderr "ERROR: daemon is not active"
   }
  } \
  "exit" {
   if { $Status } {
    event send $TmkEventII::EVT_USREVENT $TmkUsrEvent::USR_EXIT .
    puts stderr "INFO: exit event sent"
   } else {
    puts stderr "ERROR: daemon is not active"
   }
  } \
  default {
   puts stderr "Usage: $argv0"
   puts stderr " Args:        <none>  - Start daemon"
   puts stderr "              start   - Start daemon"
   puts stderr "              status  - Determine status of daemon"
   puts stderr "              restart - Terminate and restart daemon"
   puts stderr "              exit    - Safely terminate daemon"
  }
 exit
}

#
# Running in background
#

# Strip background flag from args
incr argc -1
set argv [lreplace $argv $argc $argc]

# Change our priority
if { [catch {exec setpri $Priority [pid]}] && \
     [catch {exec [file dirname [info script]]/setpri $Priority [pid]}] } {
 set Priority "default"
}

# Get Mfs device list
set fd [open "$env(MFS_DEVICE)" RDONLY]; fconfigure $fd -translation binary
seek $fd  36; set devlist [string trim [read $fd 128] "\000"]
seek $fd 196; binary scan [read $fd 4] I MFS_Zone
close $fd

set MFS_DevCnt 0
foreach {dev} $devlist {

 # Get sector length for each partition
 scan "$dev" "%8s%d" basedev partition
 set devfd [open "$basedev" RDONLY]; fconfigure $devfd -translation binary
 seek $devfd [expr 512*$partition+12]; binary scan [read $devfd 4] I sec
 set MFS_DevSectors($MFS_DevCnt) [expr $sec & ~(1023)]
 close $devfd

 # Open the partition for reading
 set MFS_Dev($MFS_DevCnt) [open $dev RDONLY]
 fconfigure $MFS_Dev($MFS_DevCnt) -translation binary

 incr MFS_DevCnt
}

# Open database
set db [dbopen]

# Get our Time Zone
if { ![info exists env(TZ)] } {

 # Get time zone and daylight savings from database
 RetryTransaction {
  if { [catch {set config  [db $db open /State/LocationConfig]}] } {
   set config  [db $db open /Setup]
   set tz      [dbobj $config get TimeZone]
  } else {
   set tz      [dbobj $config get TimeZoneOld]
  }
  set dst      [dbobj $config get DaylightSavingsPolicy]
 }

 # If not found, use defaults
 if {$tz  == ""} { set tz  0 }
 if {$dst == ""} { set dst 2 }

 # Convert internal timezone format to more normal one
 switch -- $tz \
  1 - 2 - 3 - 4 - 5 - 6 {set tz [expr -($tz+4)]} \
  7 - 8 - 9 - 10 - 11 - 12 - 13 - 14 - 15 - 16 - 17 - 18 - 19 { set tz [expr $tz-7] } \
  20 - 21 - 22 - 23 { set tz [expr -($tz-19)] } \
  24 - 25 { set tz [expr -($tz-13) ] }

 # Give some common names to zones we know
 switch -- $tz \
   0      { set tzn "GMT";  set tdn "GMT"}  \
  -4      { set tzn "AST";  set tdn "ADT"}  \
  -5      { set tzn "EST";  set tdn "EDT"}  \
  -6      { set tzn "CST";  set tdn "CDT"}  \
  -7      { set tzn "MST";  set tdn "MDT"}  \
  -8      { set tzn "PST";  set tdn "PDT"}  \
  -9      { set tzn "NAST"; set tdn "NADT"} \
  -10     { set tzn "HST";  set tdn "HDT"}  \
  default { set tzn "UCT";  set tdn ""}

 # Set time zone name and TZ variable from given information
 set tzn "$tzn[expr -$tz]"
 if { $dst == 2 } { set tzn "$tzn$tdn" }
 set env(TZ) $tzn
}

# Open osdcons fifo
set cons [open $OSD {WRONLY NONBLOCK}]

# Register for  events
event register $TmkEvent::EVT_MW_STATUS  EventCallback
event register $TmkEventII::EVT_USREVENT EventCallback

# We are ready to go
puts stderr "[clock format [clock seconds] -format "%D %T"] $argv0 $Prog $Vers Started (Priority $Priority)"

while {!$Exit} {
 set EventWait -1; vwait EventWait
 if { $Exit || $CurrentContext != $nowShowing } { continue }

 # Reset the console
 puts -nonewline $cons $init

 if { $PreviousContext == $TivoCentral || \
       [expr $DataAge+$Reload_Interval] < [clock seconds] } {
  # Init variables
  set DataAge       0
  set Spc_Total     0
  set Spc_Available 0
  set Spc_Deleted   0
  set Spc_Suggest   0
  set Spc_Perm      0
  set Spc_Recording 0
  set Cnt_Deleted   0
  set Cnt_Suggest   0
  set Cnt_Perm      0
  set Cnt_Recording 0
  set InProgress    { }

  puts -nonewline $cons "\033\133H     Getting Data.\033\133J"
  flush $cons

# Calculate MFS zone sizes for media regions

  # First zone pointer
  set next $MFS_Zone

  while { $next != 0 } {

   # Find correct partition to read zone info
   for { set x 0 } { $x < $MFS_DevCnt } { incr x } {
    if { $next < $MFS_DevSectors($x) } { break }
    incr next [expr -$MFS_DevSectors($x)]
   }

   # If zone type == 2 (MFS media) then increment counters
   seek $MFS_Dev($x) [expr 512*$next+32]
   binary scan [read $MFS_Dev($x) 4] I type
   if { $type == 2 } {
    seek $MFS_Dev($x) [expr 512*$next+52]
    binary scan [read $MFS_Dev($x) 4] I size
    seek $MFS_Dev($x) [expr 512*$next+60]
    binary scan [read $MFS_Dev($x) 4] I free
    incr Spc_Total     $size
    incr Spc_Available $free
   }

   # Next zone
   seek $MFS_Dev($x) [expr 512*$next+12]
   binary scan [read $MFS_Dev($x) 4] I next
  }

  # Convert space figures to KiB
  set Spc_Total     [expr $Spc_Total/2]
  set Spc_Available [expr $Spc_Available/2]

  update; if { $Exit || $CurrentContext != $nowShowing } { continue }

  puts -nonewline $cons ".\014"
  flush $cons

# Determine Recordings in progress

  ForeachMfsFile fsid name type "/Recording/InProgress" "" {
   RetryTransaction {
    set recording     [db $db openid $fsid]
    set selectiontype [dbobj $recording get SelectionType]
    if { $selectiontype != 0 } {
     set expirationdate [dbobj $recording get ExpirationDate]
     if { [catch { set recordingbehavior [dbobj $recording get RecordingBehavior]}] } {
      set presentationbehavior $selectiontype
     } else {
      set presentationbehavior [dbobj $recordingbehavior get PresentationBehavior]
     }
     if     { $presentationbehavior == 8 } { lappend InProgress "h" } \
     elseif { $presentationbehavior == 6 } { lappend InProgress "s" } \
     elseif { $expirationdate == 24855   } { lappend InProgress "p" } \
     else                                  { lappend InProgress "r" }
    }
   }
  }

  update; if { $Exit || $CurrentContext != $nowShowing } { continue }

  puts -nonewline $cons ".\014"
  flush $cons

# Calculate MFS space used by recordings

  # Look at each active stream
  ForeachMfsFile fsid name type "/Recording/DiskUsed/10" "" {
   RetryTransaction {
    set recording            [db $db openid $fsid]
    set state                [dbobj $recording get State]
    set selectiontype        [dbobj $recording get SelectionType]
    set streamfilesize       [dbobj $recording get StreamFileSize]
    set expirationdate       [dbobj $recording get ExpirationDate]
    if { [catch { set recordingbehavior [dbobj $recording get RecordingBehavior]}] } {
     set presentationbehavior $selectiontype
    } else {
     set presentationbehavior [dbobj $recordingbehavior get PresentationBehavior]
    }
   }

   if     { $state == 5 }                                            \
          { incr Cnt_Deleted;   incr Spc_Deleted   $streamfilesize } \
   elseif { $presentationbehavior == 8 }                             \
          { }                                                        \
   elseif { $presentationbehavior == 6 }                             \
          { incr Cnt_Suggest;   incr Spc_Suggest   $streamfilesize } \
   elseif { $expirationdate == 24855 }                               \
          { incr Cnt_Perm;      incr Spc_Perm      $streamfilesize } \
   else   { incr Cnt_Recording; incr Spc_Recording $streamfilesize }
  }

  update; if { $Exit || $CurrentContext != $nowShowing } { continue }

  puts -nonewline $cons ".\014"
  flush $cons

  set Spc_Overhead [max [expr $Spc_Total-$Spc_Perm-$Spc_Recording-$Spc_Suggest-$Spc_Deleted-$Spc_Available] 0]

  # Convert to percentages (round to nearest 2% for display)
  set tot [expr $Spc_Total/1024]
  set prm [expr round((50*($Spc_Perm     +0.0)/1024)/$tot)*2]
  set rec [expr round((50*($Spc_Recording+0.0)/1024)/$tot)*2]
  set sug [expr round((50*($Spc_Suggest  +0.0)/1024)/$tot)*2]
  set del [expr round((50*($Spc_Deleted  +0.0)/1024)/$tot)*2]
  set fre [expr round((50*($Spc_Available+0.0)/1024)/$tot)*2]
  set ovr [max [expr 100-$prm-$rec-$sug-$del-$fre] 0]

  # Start output string with defaults
  set dat "\033\1331;1y\033\13337;48m"

  # Build Space Graph (Overlay with % Used)
  append dat "\033\1332;6H\021"
  for { set x 0 } { $x < [expr $fre+$del+$sug+$rec+$prm+$ovr] } { incr x 2 } {
   set idx [expr [expr $x>=                    $ovr]+ \
                 [expr $x>=               $prm+$ovr]+ \
                 [expr $x>=          $rec+$prm+$ovr]+ \
                 [expr $x>=     $sug+$rec+$prm+$ovr]+ \
                 [expr $x>=$del+$sug+$rec+$prm+$ovr]]
   append dat "$Color([expr $idx+3*($x % 4)])"
   if { [expr $x % 4] != 0 } {
    append dat "\336\033\1333;4;9;37m\010 \033\13329m\010 \033\13323;24m"
   }
  }
  append dat "\033\1330;48m\020"

  # Overlay percentage on bar graph
  set    pct [expr round((100*($Spc_Overhead+$Spc_Perm+$Spc_Recording+0.0)/1024)/$tot)]
  append dat "\033\1332;15H\033\1336m[format "%3d" $pct]% Used\033\13326m"

  # Build Statistics
  set    shw [expr $Cnt_Perm+$Cnt_Recording+$Cnt_Suggest]
  append dat "\033\1331;6HNow Showing: $shw"
  if { [expr $Cnt_Perm+$Cnt_Recording+$Cnt_Suggest] > 0} {
   set pos [expr 35-[string length "$Cnt_Perm $Cnt_Recording $Cnt_Suggest"]]
   append dat "\033\1331;${pos}H\133"
   append dat "$Color(7)$Cnt_Perm $Color(8)$Cnt_Recording $Color(9)$Cnt_Suggest"
   append dat "\033\13337m\135"
  }

  # Build InProgress Indicator
  if { [expr [llength $InProgress]] > 0 } {
   append dat "\033\1332;34H"
   foreach {type} $InProgress {
    if     {$type == "h"} {append dat "\033\13341m "} \
    elseif {$type == "s"} {append dat "\033\13349m "} \
    elseif {$type == "p"} {append dat "\033\13342m "} \
    else                  {append dat "\033\13343m "}
   }
   # Surrond with red bars on top and bottom
   set Spaces [string range "   " 0 [expr [llength $InProgress]-1]]
   append dat "\033\1332;34H\033\1333;4;9;31m$Spaces"
   append dat "\033\1332;34H\033\13329m$Spaces\033\13323;24;37;48m"
  }

  set DataAge [clock seconds]

  update; if { $Exit || $CurrentContext != $nowShowing } { continue }
 }

 # Report data to user
 puts -nonewline $cons "$dat\014"
 flush $cons

}

# Close OSD fifo
puts -nonewline $cons "\033\1331;1y\033\133+p"
close $cons

# Close DataBase
dbclose $db

# Close MFS Devices
for { set x 0 } { $x < $MFS_DevCnt } { incr x } { close $MFS_Dev($x) }

# We are done
puts stderr "[clock format [clock seconds] -format "%D %T"] $argv0 $Prog $Vers Ending"

# Check for restart
if { $Restart } { exec $argv0 $BackGroundFlag >>&$OutFile & }

# Exit program normally
exit

