#!/tvbin/tivosh 

# dailymail.tcl
# e-mail TiVo status and info

# Version 0.2
# 3/11/03
# by Stuart Anderton (sanderton at tivocommunity.com)

# Some procs borrowed from TiVoWeb as you can't call them by reference, credited below
# Ta to vassilis for date fix

global version3
global reload
global dlc
global uktivo
global tzoffset
global nowshowingdir
global channeltablefsid

#set variables
set mailserver 212.159.11.46 ;# the IP address of your ISP's SMTP server
set domain you.com ;# your domain
set recipient you@youremail.co.uk ;# the email address to send to
set hoursahead 36 ;# number of hours ahead to look for enclosed info
set version3 false ;# change to true if you are on v3
set uktivo true ;# change to false if you are a colonial


#needed to fool TiVoWeb modules that they are being called by TiVoWeb :)
global reload
global dlc
set reload 1
set dlc 1
set channeltablefsid {}

setpri fifo 1
catch {EnableTransactionHoldoff true}

proc strim {str} {
	return [string trim $str "\{\}"]
}


# From TiVoWeb's ui.itcl
# Copyright (c) 2002 Aaron Schrab <aaron@schrab.com>
proc alt_showing {tofind_tmsid tofind_fsid} {
  global db

  set showings [get_programshowings $tofind_fsid 1]

  foreach showing $showings {
    set seconds [lindex $showing 0]
    set stationfsid [lindex $showing 1]
    set date [expr $seconds / 86400]
    set time [expr $seconds % 86400]
    set timestr [format "%05d" $time]
    ForeachMfsFile recfsid name type "/Recording/Active" "4:$date:$timestr:" {
      RetryTransaction {
	set rec [db $db openid $recfsid]
	set show [dbobj $rec get Showing]
	set prog [dbobj $show get Program]
	set id [dbobj $prog get TmsId]
	if {$id == $tofind_tmsid} {
	  return "todo [dbobj $show fsid]/[dbobj $show subobjid]"
	}
      }
    }
  }

  # Also check if it's in Now Showing
  ForeachMfsFile recfsid name type  "/Recording/TmsId" "$tofind_tmsid:" {
    RetryTransaction {
      set rec [db $db openid $recfsid]
      set show [dbobj $rec get Showing]
      return "nowshowing [dbobj $show fsid]/[dbobj $show subobjid]"
    }
  }

  return ""
}

# From TiVoWeb's httpd-tt.tcl by mbm
proc get_tzoffset {mfstz dst} {
   if { $mfstz <= 0 } {
      set tz $mfstz
   } else {
      set tzlist "-5 -6 -7 -8 -9 -10 0 1 2 3 4 5 6 7 8 9 10 11 12 -1 -2 -3 -4 -11 -12"
      set tz [lindex $tzlist [expr $mfstz - 1]]
   }
   if { $dst == 2 } {
      set date [clock format [clock seconds] -format "%1d %w %1m %1H %1M"]
      scan $date "%d %d %d %d %d" dom dow month hour min
      set dlsval 0
      if {$::uktivo} {
         if {$month > 3 && $month < 10} {
            set dlsval 1
         } elseif {$month == 3 && $dom >= 25 && $dow == 0 && $hour >= 1} {
            set dlsval 1
         } elseif {$month == 3 && $dom >= 25 && $dow != 0 && ($dom-24-$dow >= 1) } {
            set dlsval 1
         }
      } else {
         if {$month > 4 && $month < 10} {
            set dlsval 1
         } elseif {$month == 4 && $dom > 7} {
            set dlsval 1
         } elseif {$month == 4 && $dom <= 7 && $dow == 0 && $hour >= 2} {
            set dlsval 1
         } elseif {$month == 4 && $dom <= 7 && $dow != 0 && ($dom-$dow > 0)} {
            set dlsval 1
         }
      }
      if {$dlsval == 0} {
         if {$month == 10 && $dom < 25} {
            set dlsval 1
         } elseif {$month == 10 && $dom >= 25 && $dow == 0 && $hour < 2} {
            set dlsval 1
         } elseif {$month == 10 && $dom >= 25 && $dow != 0 && ($dom-24-$dow < 1) } {
            set dlsval 1
         }
      }
      if {$dlsval == 1} {
         return [expr ($tz+1)*60*60]
      } else {
         return [expr $tz*60*60]
      }
   } else {
      return [expr $tz*60*60]
   }
}

# sends results as an e-mail
proc sendmail {mailserver domain recipient subject message} {

	set chan [socket $mailserver 25]
	gets $chan reply
	#<server>220 [server name] [server capabilities]
	if {[lindex $reply 0] != "220"} {
		puts "Bad connection"
		exit 0
	} else {
		puts "Connected"
	}

	puts $chan "HELO $domain"

	flush $chan
		gets $chan reply
	#<server>250 [informational message, usually server name]
	if {[string range $reply 0 2] != "250"} {
		puts "Connection rejected"
		exit 0
	} else {
		
		puts "Ehlo acknowledged"
	}

	puts $chan "MAIL FROM: tivo@$domain"
	flush $chan
	gets $chan reply
	#<server>250 ok
	if {[string range $reply 0 2] != "250"} {
		puts "Connection rejected"
		exit 0
	} else {
		puts "From acknowledged"
	}

	puts $chan "RCPT TO: $recipient"
	flush $chan
	gets $chan reply
	#<server>250 ok
	if {[string range $reply 0 2] != "250"} {
		puts "Connection rejected"
		exit 0
	} else {
		puts "To acknowledged"
	}
	



	puts $chan "DATA"
	flush $chan
	gets $chan reply
	#<server>354 go ahead
	if {[string range $reply 0 2] != "354"} {
		puts $reply
		puts "Connection rejected"
		exit 0
	} else {
		puts "Sending message"
	}

	puts $chan "Subject: $subject"
	flush $chan
	puts $chan "Date: [clock format [clock seconds] -format {%a, %d %b %Y %H:%M:%S (%Z)} -gmt true]"
	flush $chan
	puts $chan "From: tivo@$domain"
	flush $chan
	puts $chan "Message-Id: [clock seconds]@$domain"
	flush $chan
	puts $chan "MIME-Version: 1.0"
	flush $chan
	puts $chan "Content-type: text/html"
	flush $chan
	puts $chan "<html> <body>"
	flush $chan
	foreach item $message {
		foreach part $item {
			foreach line $part {
				puts $chan [strim $line]		
			}
		}
	}
	flush $chan
	puts $chan "</body> </html>  "
	flush $chan

	puts $chan ""
	flush $chan
	puts $chan ""
	flush $chan
	puts $chan "."
	flush $chan
	gets $chan reply
	#<server>250 [informational message]
	if {[string range $reply 0 2] != "250"} {
		puts "Sending failed"
		exit 0
	} else {
		puts "Message sent"
	}

	puts $chan "quit"
	flush $chan
	gets $chan reply
	#<server>221 [informational message]

	if {[string range $reply 0 2] != "221"} {
		puts "Disconnect failed"
		exit 0
	} else {
		puts "Disconnected"
	}

	close $chan

}

# get unresolved conflicts
proc getconflicts {hoursahead} {
	
	global db
	global tzoffset
	set conflicts {}
	
	ForeachMfsFile fsid name type "/Recording/History" "" {
			
			set recdate [split $name ":"]
			regsub {^(-?)0+([1-9])} [lindex $recdate 1] {\1\2} thetime
			set rectime [expr [lindex $recdate 0] * 86400 + $thetime]
	
			if {$rectime > [expr [clock seconds] + $hoursahead * 3600]} {
				break
			}
		
			if {$rectime > [clock seconds]} {
	
				RetryTransaction {
					set rec [db $db openid $fsid ]
					set creason [dbobj $rec get CancelReason]
					set showing [dbobj $rec get Showing]
					set station [dbobj $showing get Station]
					set callsign [dbobj $station get CallSign]
					set program [dbobj $showing get Program]
					set title [strim [dbobj $program get Title]]
					set id [dbobj $program get TmsId]
					set id2 [dbobj $program fsid]
					set descr [strim [dbobj $program get Description]]
					set series [dbobj $program get Series]


					if {$series != ""} {
					set episodic [defaultval 1 [dbobj $series get Episodic]]
					} else {
					set episodic 1
					}

					if { $episodic == 1 } {
					set eptitle [strim [dbobj $program get EpisodeTitle]]
					if { $eptitle == "" } {
					set eptitle "No Episode Title"
					}
					} else {
					set eptitle ""
					}
					}
				
	
				if {$creason == 20} {
					
					set alt [alt_showing $id $id2]
					if {$alt == ""} {
						set conflict {}
						lappend conflict "<b>$title</b>" 
						lappend conflict "<i>[clock format [expr $rectime + $tzoffset] -format {%a %H:%M}] $callsign</i>"
						lappend conflict "$eptitle - $descr <p>"
						lappend conflicts $conflict
				}

				}
			}
		}
	
	
	

	
	return $conflicts
}

proc gettodo {hoursahead} {
	
	global db
	global tzoffset
	set message {}


	set path "/Recording/Active"
	set prefix "4"

	ForeachMfsFileTrans fsid name type $path $prefix 10 {

		set rec [db $db openid $fsid]
		set showing [dbobj $rec get Showing]
		set starttime [expr [dbobj $showing get Date] * 86400 + [dbobj $showing get Time]]

		if {$starttime > [expr ($hoursahead *3600) + [clock seconds]]} {
			return $message
		}

		set showingfsid [dbobj $rec gettarget Showing]
		set station [dbobj $showing get Station]
		set stationfsid [dbobj $station fsid]
		set callsign [dbobj $station get CallSign]
		set program [dbobj $showing get Program]
		set title [strim [dbobj $program get Title]]
		set descr [strim [dbobj $program get Description]]


		set series [dbobj $program get Series]


		if {$series != ""} {
		set episodic [defaultval 1 [dbobj $series get Episodic]]
		} else {
		set episodic 1
		}

		if { $episodic == 1 } {
		set eptitle [strim [dbobj $program get EpisodeTitle]]
		if { $eptitle == "" } {
		set eptitle "No Episode Title"
		}
		} else {
		set eptitle ""
		}

		set text {}
		lappend text "<b>$title</b>" 
		lappend text "<i>[clock format [expr $starttime + $tzoffset] -format {%a %H:%M}] $callsign</i><br>"
		lappend text "$eptitle - $descr <p>"
		lappend message $text

	}
	
	return $message

}
	


proc getnpexpire {hoursahead} {
	
	global nowshowingdir
	global tzoffset
	global db
	set message {}

	ForeachMfsFileTrans fsid name type $nowshowingdir "" 10 {
		set rec [db $db openid $fsid]	
		set expdate [dbobj $rec get ExpirationDate]
		set expsecs [expr $expdate * 86400 + [dbobj $rec get ExpirationTime]]

		if {$expsecs < [expr $hoursahead * 3600 + [clock seconds]] && $expsecs > [clock seconds] } {
			set showing [dbobj $rec get Showing]
			set startdate [dbobj $rec get StartDate]
			set starttime [dbobj $rec get StartTime]
			set rectime [expr $startdate * 86400 + $starttime]
			set station [dbobj $showing get Station]
			set stationfsid [dbobj $station fsid]
			set callsign [dbobj $station get CallSign]
			set program [dbobj $showing get Program]
			set title [strim [dbobj $program get Title]]
			set descr [strim [dbobj $program get Description]]
			set series [dbobj $program get Series]

			if {$series != ""} {
				set episodic [defaultval 1 [dbobj $series get Episodic]]
			} else {
				set episodic 1
			}

			if { $episodic == 1 } {
				set eptitle [strim [dbobj $program get EpisodeTitle]]
			if { $eptitle == "" } {
				set eptitle "No Episode Title"
			}
			} else {
				set eptitle ""
			}

			set text {}
			lappend text "<b>$title</b>" 
			lappend text "<i>Recorded [clock format [expr $rectime + $tzoffset] -format {%a %H:%M}] $callsign</i>"
			lappend text "<i> Expires [clock format [expr $expsecs + $tzoffset] -format {%a %H:%M}]</i><br>"
			lappend text "$eptitle - $descr <p>"
			lappend message $text	
		}



	}

	return $message

}

# adapted from the phone.itcl module by Courtney Bailey
proc phone_status {} {
	
	global db
	global version3
	
	set errormessages { "Blank" "Succeeded" "PendingRestart" \
				"InitializationFailure" "SettingUp" "InProgress" \
				"UnknownError" "Failure during Housekeeping" \
				"Failure during Preparation" "Failed. Line unavailable" \
				"Failed. No dial tone" "Failed. Couldn't dial" "Failed. Wrong number" \
				"Failed. Service is not answering" "Failed. Service unavailable" \
				"Failed. Service denied" "Failed. Call interrupted" \
				"Failed while expanding data" "Failed while preparing data" \
				"Failed while loading series" "Failed while indexing shows" "Failed to set clock" \
				"Failed getting account status" "Failure downloading messages" "Failed. Phone is busy" \
				"Failed while indexing days" "Failed getting account status" "In Progress" \
				"Housekeeping" "Preparing to call" "Dialing..." "Answering..." "Answered" \
				"Connecting..." "Getting account status" "Setting clock..." "Downloading messages" \
				"Processing..." "Downloading..." "Hanging up" "Expanding data" "Preparing data" \
				"Loading data" "Indexing series" "Indexing shows" "Indexing days" \
				"Failed. Modem is not responding" "Configuring modem" "Failed. No dial-in number chosen." \
				"Finishing up..." }
	
	  if {$::version3} {
		  set config "/State/PhoneConfig"
		  set serviceinfo $config
	  } else {
		  set config "/Setup"
		  RetryTransaction {
			  set configdir [db $db open $config]
			  set serviceinfo [dbobj $configdir get ServiceInfo]
		  }
	  }
	  
	RetryTransaction {
		set configdir [db $db open $config]
		
		if {$::version3} {
			set serviceinfo $configdir
		} else {
			set serviceinfo [dbobj $configdir get ServiceInfo]
		}
		
		set lastcallstatus [dbobj $configdir get LastCallStatus]
		if { $lastcallstatus == "Failed"} {
			set csi [dbobj $serviceinfo get CallStatusInfo]
			if {$csi != "Failed" } {
				regexp {[0-9]+} $csi lastcallstatus
				set lastcallstatus [lindex $errormessages $lastcallstatus]
			}
			return $lastcallstatus
		} else {
			set lastcallstatus "Ok"
			return $lastcallstatus
		}
	}
}

#############################################
# main code

# open the MFS database
global db
set db [dbopen]

# call procedures in TiVoWeb to do the heavy lifting
source /var/hack/tivoweb-tcl/util.itcl
source /var/hack/tivoweb-tcl/modules/index.itcl

if {$version3} {
      set nowshowingdir "/Recording/NowShowingByClassic"
   } else {
      set nowshowingdir "/Recording/NowShowing"
   }

# set timezone offset

RetryTransaction {
if {$::version3} {
      set lconfig  [db $db open /State/LocationConfig]
      set setup [db $db open /State/ServiceConfig]
      set tz [dbobj $lconfig get TimeZoneOld]
      set summertime [dbobj $lconfig get DaylightSavingsPolicy]
   } else {
	set setup [db $db open /Setup]
	set tz [dbobj $setup get TimeZone]
      set summertime [dbobj $setup get DaylightSavingsPolicy]
   } ;[base64dec]
   if {$tz == ""} {
      set tz 0
   }
   if {$summertime == ""} {
      set summertime 2
   }

}
set tzoffset [get_tzoffset $tz $summertime]

# Assemble e-mail
set message {}

set text "<h3>Welcome to the TiVo status e-mail for [clock format [clock seconds] -format %A]</h3>"
lappend message $text

# how long since last reboot
set uptime [lindex [exec cat /proc/uptime] 0]

if {$uptime < 86400} {
	set text "WARNING: The TiVo has rebooted within the last 24hrs.<p>"
	lappend message $text
}

set phone [phone_status]
if {$phone != "Ok"} {
	set text "WARNING: Last daily call failed with error of $phone <p>"
	lappend message $text
}
	


set todo [gettodo $hoursahead]
if {$todo != ""} {
	set text "<b>The following programmes are scheduled to record in the next $hoursahead hours: </b><p>"
	lappend message $text
	lappend message $todo
} else {
	set text "<b>No recordings scheduled in the next $hoursahead hours</b><p>"
	lappend message $text
}

set expires [getnpexpire $hoursahead]
if {$expires != ""} {
	set text "<b>The following Now Playing entries expire in the next $hoursahead hours: </b><p>"
	lappend message $text
	lappend message $expires
} else {
	set text "<b>No Now Playing entries expire in the next $hoursahead hours</b><p>"
	lappend message $text
}

set conflicts [getconflicts $hoursahead]
if {$conflicts != ""} {
	set text "<b>The following unresolved conflicts occur in the next $hoursahead hours: </b><p>"
	lappend message $text
	lappend message $conflicts
} else {
	set text "<b>No unresolved conflicts occur in the next $hoursahead hours</b><p>"
	lappend message $text
}

dbclose $db

sendmail $mailserver $domain $recipient "Daily status e-mail" $message
