#!/tvbin/tivosh
#2003 embeem

class readfile {
   private variable fd ""
   constructor {filename} {
     set fd [open $filename]
     fconfigure $fd -translation binary
   }
   destructor {
     close $fd
   }
   method read {bytes} {
     return [::read $fd $bytes]
   }
   method fd {} {
     return $fd
   }
}

class readtar {
  private variable eof              1
  private variable length           0
  private variable remainder        0
  private variable obj              ""
  constructor {_obj} {
    set obj $_obj
  }
  method read {bytes} {
    if {$bytes >= $length || $bytes == -1} {
      set bytes $length
      set eof 1
    }
    incr length    -$bytes
    incr remainder -$bytes
    set  remainder [expr $remainder%512]
    return [$obj read $bytes]
  }
  method next {} {
    $obj read $remainder
    set eof       0
    set remainder 0
    set name      ""
    set magic     ""

    set header [$obj read 512]
    scan [string range $header 0   100] "%s" name
    scan [string range $header 124 135] "%o" length
    scan [string range $header 257 262] "%s" magic
    if {$magic=="ustar"} {
      return $name
    }
    return ""
  }
  method eof {} {
    return $eof
  }
  method length {{set 0}} {
    if {!$set} {
      return $length
    } else {
      if {-$set >= $length} {
        set eof 1
      }
      incr length $set
      incr remainder $set
      set  remainder [expr $remainder%512]

    }
  }
}

proc xml2tcl {xml} {
  set stack ""
  set objstack ""
  set res ""

  while {[regexp {<([^>]*)>(([^<]*).*)} $xml dummy tag xml val]} {
    switch [string index $tag 0] {
      ? {
      }
      / { ;# end tag
        set tag [string range $tag 1 end]
        if { $tag == [lindex $stack end] } {
          set stack [lrange $stack 0 [expr [llength $stack]-2]]; #pop tag off stack
          switch $tag {
            Object -
            SubObject { set objstack [lrange $objstack 0 [expr [llength $objstack]-2]]}
          }
        } else {
          puts "XML: missing endtag? $tag"
        }
      }
      default {
        lappend stack [lindex $tag 0]
        set parent [lindex $objstack end]
        if {[regexp {(.*) type=\"(.*)\" id=\"(.*)\"} $tag dummy obj type id]} {
          switch $obj {
            Object {
              set myobj [db $::db create $type]
              if {$type=="Recording"} {
                set res $myobj
              }
            }
            SubObject {
              set myobj [db $::db createsub $type $parent]
              if {$type=="RecordingPart"} {
                dbobj $myobj set File 0 ;# placeholder
                lappend res $myobj
              }
            }
          }
          if {$id!="_top"} {
            dbobj $parent add $id $myobj
          }
          lappend objstack $myobj
        } elseif {$tag!="ServerId"} {
          dbobj $parent add $tag $val
        }
      }
    }
  }
  return $res
}

proc abort {} {
        global db
        puts "ABORT -- trying to cleanup!"
      catch {
        RetryTransaction {
          set fsid [lindex $::recfsid 0]
          set sub  [lindex $::recfsid 1]
          set recording [db $db openidconstruction $fsid $sub]
          dbobj $recording markasrubbish
          foreach part $::parts {
            set fsid [lindex $part 0]
            set sub  [lindex $part 1]
            set part [db $db openidconstruction $fsid $sub]
            dbobj $part markasrubbish
            set file [dbobj $part get File]
            if {$file!=0} {
              dbunload $db $file
            }
          }
        }
      }
        exit 1
}

set db [dbopen]

readfile fileobj [lindex $argv 0]
readtar tarobj fileobj

while {[set name [tarobj next]]!=""} {
  switch -glob $name {
    *.xml {
      set len [tarobj length]
      puts "xml: $name $len"
      set xml [tarobj read $len]
      RetryTransaction {
        set res [xml2tcl $xml]
        set recording [lindex $res 0]
        dbobj $recording set BitRate 0 ;# obsolete
        dbobj $recording set ErrorString "insert"
        dbobj $recording set Score 0
        dbobj $recording set SelectionType 3
        dbobj $recording set State 4 ;# complete
        set today [expr [clock seconds] / 86400]
        set now   [expr [clock seconds] % 86401 + 10]
        dbobj $recording set ExpirationDate [expr $today + 2]
        dbobj $recording set ExpirationTime $now

        set ::recfsid [list [dbobj $recording fsid] [dbobj $recording subobjid]]
        puts "recording object: $::recfsid"
        foreach part  [lrange $res 1 end] {
          lappend ::parts [list [dbobj $part fsid] [dbobj $part subobjid]]
        }
      }
      catch {
        RetryTransaction {
          set fsid [lindex $::recfsid 0]
          set sub  [lindex $::recfsid 1]
          set recording [db $db openidconstruction $fsid $sub]
          set behavior [db $db createsub RecordingBehavior $recording]
          dbobj $behavior  set DiskBehavior         7
          dbobj $behavior  set PresentationBehavior 1
          dbobj $behavior  set ProgramGuideBehavior 1
          dbobj $behavior  set TunerBehavior        1
          dbobj $recording set RecordingBehavior $behavior
        }
      }
      event send $TmkEvent::EVT_DATA_CHANGED 2 0
    }
    *.ty {
      set len [expr [tarobj length] / 131072]
      puts "ty: $name $len / 131072"
      set fsid [lindex [lindex $::parts 0] 0]
      set sub  [lindex [lindex $::parts 0] 1]
      puts "RecordingPart $fsid/$sub"
      catch {
        RetryTransaction {
          set ty [mfs streamcreate 131072 $len]
          set part [db $db openidconstruction $fsid $sub]
          dbobj $part set File $ty
        }
      } error
      if { $error != "" } {
        puts "error allocating stream"
        abort
      }
      puts "importing ... (tystream: $ty)"
      event send 19 2 [binary format I* [pid]]

      set ::parts [lrange $::parts 1 end]
      
      set p [open "|mfs_import $ty - @>stdout" w]
      for {set x 0} {$x < $len} {incr x} {
        if {[catch {fcopy [fileobj fd] $p -size 131072}]} {
          puts "mfs_import error"
          abort
        }
        tarobj length -131072
      }
      catch {close $p}
    }
  }
  while {![tarobj eof]} {
    tarobj read 81920
    puts -nonewline "."
    flush stdout
  }
}
