#!/tvbin/tivosh

# loadlogos.tcl - command line batch loading adaptation of logos.itcl
# Original by Jake B & Joshua Faust
# Adaptation by Stuart Anderton (sanderton at TC & DD)
# v 1.1

proc strim {str} {
  return [string trim $str "\{\}"]
}

proc find_logomatch {callsign name} {
  global logotableindex logotablename
  set callsign [string tolower $callsign]
  foreach type {1 2} {
    set index [lsearch $logotablename($type) "$callsign-tw"]
    if {$index != -1} {return [lindex $logotableindex($type) $index]}
  }
  return -1
}

proc defaultval {val1 val2} {
  if { $val2 != "" } {
    return $val2
  } else {
    return $val1
  }
}

proc removeLogoIndex {sindex types rmimage} {
  global db
  global logotableindex logotablename
  
  foreach type $types {
    foreach lspace {1 2} {
      set index [expr ($lspace << 16) | $sindex]
      RetryTransaction {
        set logogroup [db $db open "/LogoGroup/$lspace:$type:$type"]
        set indexes [dbobj $logogroup get Index]
        set iindex [lsearch $indexes $sindex]
        if {$iindex != -1} {
          if {$rmimage} {
            set img [dbobj $logogroup get Image $iindex]
            set serverid [dbobj $img get ServerId]
            if {$serverid != ""} {
              error "Image has ServerId"
            }
            dbobj $img markasrubbish
          }
          dbobj $logogroup removeat Index $iindex
          dbobj $logogroup removeat Image $iindex
        }
      }
      set iindex [lsearch $logotableindex($type) $index]
      if {$iindex != -1} {
        set logotablename($type) [lreplace $logotablename($type) $iindex $iindex]
        set logotableindex($type) [lreplace $logotableindex($type) $iindex $iindex]
      }
    }
  }
}

proc init_channelindex {} {
  global db
  global guideindexdir
  global channeltablefsid
  global channeltablestation
  global channeltablenum
  global channeltableindex
  global logotableindex
  global logotablename
  
  if {[info exists logotableindex1] == 0} { init_logoindex }
  
  if {[info exists channeltablestation]} {
    unset channeltablestation
    unset channeltablenum
  }
  if {[info exists channeltableindex]} {
    unset channeltableindex
  }
  
  RetryTransaction {
    set setup [db $db open /Setup]
    set sources [dbobj $setup get Source]
    set channels ""
    foreach source $sources {
      append channels [dbobj $source gettarget Channel] " "
    }
    set channelcount [llength $channels]
  }
  set increment 30
  
  for {set j 0} {$j < $channelcount} {incr j $increment} {
    set channels2 [lrange $channels $j [expr $j + $increment - 1]]
    RetryTransaction {
      foreach channel $channels2 {
        regexp {([0-9]*)/(.*)} $channel junk id subid
        set channelobj [db $db openidconstruction $id $subid]
        set record [defaultval 1 [dbobj $channelobj get Record]]
        set favorite [defaultval 0 [dbobj $channelobj get Favorite]]
        if { $record > 0 } {
          set num [dbobj $channelobj get Number]
          set num [expr $num & 0xFFFF]
          set station [dbobj $channelobj get Station]
          set stationfsid [dbobj $station fsid]
          set callsign [dbobj $station get CallSign]
          set stationname [dbobj $station get Name]
          set logoindex [dbobj $station get LogoIndex]
          set channeltablenum($num) $stationfsid
          set channeltablestation($stationfsid) [list $num $favorite $callsign $stationname $logoindex]
        }
      }
    }
  }
  
  RetryTransaction {
    set channeltablefsid [lindex [mfs find $guideindexdir/ChannelTable] 0]
    set channeltabledata [mfs get $channeltablefsid]
  }
  
  binary scan [string range $channeltabledata 0 3] I numkeys
  for {set i 0} {$i < $numkeys} {incr i} {
    binary scan [string range $channeltabledata [expr $i*8 + 4] [expr $i*8 + 11]] ISS stationfsid dummy chnum
    if { [catch {set test $channeltablenum($chnum)}] == 0 } {
      set channeltableindex($i) $stationfsid
    }
  }
}

proc init_logoindex {} {
  global db
  global logotableindex
  global logotablename
  
  foreach logotype {1 2} {
    set logotableindex($logotype) ""
    set logotablelogo($logotype) ""
    set logotablename($logotype) ""
  }
  
  set LogoGroups ""
  ForeachMfsFile fsid name type "/LogoGroup" "" {
    lappend LogoGroups $fsid
  }
  
  #fill logotablelogo and logotableindex
  foreach fsid $LogoGroups {
    RetryTransaction {
      set LogoGroupEntry [db $db openid $fsid]
      set LogoGroupSpace [dbobj $LogoGroupEntry get LogoSpace]
      set LogoGroupSize [dbobj $LogoGroupEntry get Size]
      set LogoGroupImages [dbobj $LogoGroupEntry gettarget Image]
      set LogoGroupIndexes [dbobj $LogoGroupEntry get Index]
      
      foreach targ $LogoGroupImages ndx $LogoGroupIndexes {
        set EncodedIndex [expr ($LogoGroupSpace << 16) | ($ndx & 0xffff)]
        lappend logotableindex($LogoGroupSize) $EncodedIndex
        lappend logotablelogo($LogoGroupSize) $targ
      }
    }
  }
  
  #fill the logotablename
  foreach logotype {1 2} imgsuffix {"-s1-p1" "-s2-p2"} {
    for {set x 0} {$x < [llength $logotablelogo($logotype)]} {incr x 50} {
      RetryTransaction {
        foreach fsid [lrange $logotablelogo($logotype) $x [expr $x+49]] {
          regexp {([0-9]*)/(.*)} $fsid junk id subid
          set logo [db $db openidconstruction $id $subid]
          set name [dbobj $logo get Name]
          regsub -- "$imgsuffix\$" $name {} name
          lappend logotablename($logotype) $name
        }
      }
    }
  }
}


proc import_file {fd size} {
  if {$size < 4096} {
    set fdata [read $fd $size]
    set fsid [mfs put $fdata]
  } else {
    set start 0
    set incrsize 2048
    set fsid [mfs allocate $size]
    while {$start < $size} {
      if {[expr $start + $incrsize] > $size} {
        set fdata [read $fd [expr $size - $start]]
      } else {
        set fdata [read $fd $incrsize]
      }
      mfs putpart $fsid $start $fdata
      incr start $incrsize
    }
  }
  return $fsid
}

proc check_validpng {fd tivo logo} {
  set tmpdata [read $fd 8]
  if {[string compare $tmpdata [binary format H* 89504E470D0A1A0A]] != 0} {
    return 0
  }
  set hasheader 0
  set haspalette 0
  while {![eof $fd] } {
    set tmpdata [read $fd 8]
    binary scan $tmpdata I hlen
    set header [string range $tmpdata 4 7]
    switch -exact $header {
      "IHDR" {
        if {$hlen != 13} {return 0}
        set tmpdata [read $fd $hlen]
        if {$logo} {
          binary scan [string range $tmpdata 0 3] I width
          binary scan [string range $tmpdata 4 7] I height
          if {$width == 65 && $height == 55} {
            set type 1
          } elseif {$width == 100 && $height == 35} {
            set type 2
          } else {
            return 0
          }
        }
        if {$tivo} {
          if {[string compare [string range $tmpdata 8 12] [binary format H* 0803000000]] != 0} {
            return 0
          }
        }
        set hasheader 1
      }
      "PLTE" {
        if {$tivo} {
          if {$hlen < 765} {
            return 0
          }
        }
        seek $fd $hlen current
        set haspalette 1
      }
      default {
        seek $fd $hlen current
      }
    }
    if {!$hasheader} {
      return 0
    } elseif {!$tivo} {
      break
    } elseif {$haspalette} {
      break
    }
    seek $fd 4 current
  }
  
  if {$tivo && !$haspalette} {
    return 0
  }
  if {$logo} {
    return $type
  } else {
    return 1
  }
  
}


proc get_fsidbyprefix {dirname prefix} {
  if { [catch {mfs scan $dirname -start $prefix -count 1} batch] } {
    global errorCode errorInfo
    if { $errorCode == "errNmNameNotFound" } {
      return
    } else {
      error $batch $errorInfo $errorCode
    }
  }
  set item [lindex $batch 0]
  set name [lindex $item 1]
  if { [PrefixMatches $prefix $name] } {
    return $item
  } else {
    return ""
  }
}

proc lsearchall {llist pattern} {
  set rlist ""
  set index [lsearch $llist $pattern]
  set index1 $index
  while {$index1 != -1} {
    lappend rlist $index
    set index1 [lsearch [lrange $llist [expr $index + 1] end] $pattern]
    set index [expr $index + $index1 + 1]
  }
  if {$rlist == ""} {
    return -1
  } else {
    return $rlist
  }
}

proc show_import {file} {
  global db
  global source_dir
  global logotableindex logotablename
  set lspace 1 ;# change to 2 for DirecTivos
  
  
  
  
  if {$file == ""} {
    puts "Error: No File Specified"
    return
  }
  set filepath $file
  set filelist [split $file "/"]
  set file [lindex $filelist end]
  if { [catch {
      set fsize [file size $filepath]
      set fd [open $filepath "r"]
    }] == 1} {
    puts "Error Opening File $file"
    return
  }
  if {$fsize == 0} {
    puts "Error: File Empty $file"
    close $fd
    return
  }
  fconfigure $fd -translation binary
  set type [check_validpng $fd 1 1]
  # if {[regexp -- {-s1} $file]} {
  #	  set type 1
  # } else {
  #	  set type 2
  # }
  seek $fd 0 start
  set file [string tolower $file]
  regsub ".png$" $file {} fname
  if {$type == 0} {
    puts "Error: Invalid file format $file"
    close $fd
    return
  } elseif {$type == 1} {
    if {[regexp -- {-s1-p1$} $fname]} {
      if {![regexp -- {-tw-s1-p1$} $fname]} {
        regsub -- {-s1-p1$} $fname {-tw-s1-p1} fname
      }
      regsub -- {-s1-p1$} $fname {-s2-p2} fnamealt
      regsub -- {-s1-p1$} $fname {} fnamebase
    } else {
      if {[regexp -- {-s2-p2$} $fname]} {
        puts "Error: Filename suffix '-s2-p2' does not match detected type '1'$file"
        close $fd
        return
      }
      append fname "-tw"
      set fnamebase $fname
      append fname "-s1-p1"
      set fnamealt "$fname-s2-p2"
    }
  } elseif {$type == 2} {
    if {[regexp -- {-s2-p2$} $fname]} {
      if {![regexp -- {-tw-s2-p2$} $fname]} {
        regsub -- {-s2-p2$} $fname {-tw-s2-p2} fname
      }
      regsub -- {-s2-p2$} $fname {-s1-p1} fnamealt
      regsub -- {-s2-p2$} $fname {} fnamebase
    } else {
      if {[regexp -- {-s1-p1$} $fname]} {
        puts "Error: Filename suffix '-s1-p1' does not match detected type '2'$file"
        close $fd
        return
      }
      append fname "-tw"
      set fnamebase $fname
      append fname "-s2-p2"
      set fnamealt "$fname-s1-p1"
    }
  }
  set alttype "G1s/N2wbWz8yNWwbWzU7MTByG1soMBtbPzloCg=="
  puts "Detected Type $type Palette/Size Format fro $file"
  RetryTransaction {
    set existingimg [lindex [get_fsidbyprefix "/Resource/Image" $fname] 0]
    if {$existingimg != ""} {
      set obj [db $db openid $existingimg]
      puts "Overwriting Existing Image for $file"
      set ifsid [import_file $fd $fsize]
      dbobj $obj set File $ifsid
      set imgfsid [dbobj $obj fsid]
    } else {
      set ifsid [import_file $fd $fsize]
      set obj [db $db create Image]
      dbobj $obj set File $ifsid
      dbobj $obj set Format 2
      dbobj $obj set Name $fname
      set imgfsid [dbobj $obj fsid]
    }
  }
  close $fd
  puts "Successfully Imported Image $file"
  
  set alttype [expr $type == 1 ? 2 : 1]
  set indexes [lsearchall $logotablename($type) $fnamebase]
  set index ""
  set indexals ""
  if {$indexes != -1} {
    foreach index1 $indexes {
      set index2 [lindex $logotableindex($type) $index1]
      if {[expr $index2 >> 16] == $lspace} {
        set index $index2
      } else {
        set indexals $index2
      }
    }
  }
  set altindexes [lsearchall $logotablename($alttype) $fnamebase]
  set altindex ""
  set altindexals ""
  if {$altindexes != -1} {
    foreach index1 $altindexes {
      set index2 [lindex $logotableindex($alttype) $index1]
      if {[expr $index2 >> 16] == $lspace} {
        set altindex $index2
      } else {
        set altindexals $index2
      }
    }
  }
  set altlspace [expr $lspace == 1 ? 2 : 1]
  if {$index == ""} {
    if {$altindex != ""} {
      set index [lsearch $logotableindex($type) $altindex]
      if {$index != -1} {
        puts "Error: Sister Logo Exists by Name, but Does Not Match Index for $file"
        return
      } else {
        addLogoIndex $imgfsid $altindex
        puts "Associating $file with Sister Logo $fnamealt.png"
      }
    } elseif {$indexals != ""} {
      set indexals [expr ($lspace << 16) | ($indexals & 0xffff)]
      set index [lsearch $logotableindex($type) $indexals]
      if {$index != -1} {
        puts "Error: $file Exists by Name in the Other Logo Space, but Does Not Match Index"
        return
      } else {
        addLogoIndex $imgfsid $indexals
        puts "Associating $file with Logo from the Other Logo Space"
      }
      
    } elseif {$altindexals != ""} {
      set altindexals [expr ($lspace << 16) | ($altindexals & 0xffff)]
      set index [lsearch $logotableindex($type) $altindexals]
      if {$index != -1} {
        puts "Error: Sister Logo to $file Exists by Name in the Other Logo Space, but Does Not Match Index"
        return
      } else {
        addLogoIndex $imgfsid $altindexals
        puts "Associating $file with Sister Logo from the Other Logo Space $fnamealt.png"
      }
    } else {
      set count 3999
      set empty 0
      while {$empty != 1} {
        set empty 1
        incr count
        foreach lspace2 {1 2} {
          set index [expr ($lspace2 << 16) | $count]
          foreach type2 {1 2} {
            if {[lsearch $logotableindex($type2) $index] != -1} {
              set empty 0
            }
          }
        }
      }
      set index [expr ($lspace << 16) | $count]
      addLogoIndex $imgfsid $index
      puts "<Associating $file as New Logo"
    }
  } else {
    puts "$file Already Associated as Logo,"
    #removeLogoIndex $index $type 1
    #set count 3999
    #set empty 0
    #while {$empty != 1} {
    #set empty 1
    #incr count
    #foreach lspace2 {1 2} {
    #set index [expr ($lspace2 << 16) | $count]
    #foreach type2 {1 2} {
    #if {[lsearch $logotableindex($type2) $index] != -1} {
    #set empty 0
    #}
    #}
    #}
    #}
    #set index [expr ($lspace << 16) | $count]
    #addLogoIndex $imgfsid $index
  }
  
}


proc addLogoIndex {imgfsid index} {
  global db
  global logotableindex logotablename
  
  set lspace [expr $index >> 16]
  RetryTransaction {
    set img [db $db openid $imgfsid]
    set name [dbobj $img get Name]
    if {[regsub -- {-s1-p1$} $name {} name] == 1} {
      set type 1
    } elseif {[regsub -- {-s2-p2$} $name {} name] == 1} {
      set type 2
    } else {
      error "Invalid image name suffix"
    }
    set logogroup [db $db open "/LogoGroup/$lspace:$type:$type"]
    dbobj $logogroup add Index [expr $index & 0xffff]
    dbobj $logogroup add Image $img
  }
  lappend logotablename($type) $name
  lappend logotableindex($type) $index
}

proc setLogoIndex {fsid logoindex} {
  global db
  global channeltablestation
  
  #first set the mfs entry to the new logoindex.
  RetryTransaction {
    set station [db $db openid $fsid]
    set type [dbobj $station type]
    if {$type == "Station"} {
      if {$logoindex != ""} {
        dbobj $station set LogoIndex $logoindex
      } else {
        dbobj $station remove LogoIndex
      }
    } else {
      error "Invalid object type for setting LogoIndex"
    }
  }
  
  set channeltablestation($fsid) [lreplace $channeltablestation($fsid) 4 4 $logoindex]
}

global db
global source_dir
global logotableindex logotablename guideindexdir channeltablenum channeltablestation

set source_dir [lindex $argv 0]

if {$source_dir == ""} {
  puts "loadlogos.tcl Usage:"
  puts "loadlogos.tcl dir"
  puts "dir - directory containing logos to load"
  exit 0
}

if {![file exists $source_dir]} {
  puts "Directory $source_dir not found"
  exit 0
}

set guideindexdir "/GuideIndex"


set filelist [glob -nocomplain $source_dir/*.png]

if {$filelist == ""} {
  puts "No .png files found"
}

set db [dbopen]
puts "Initialising logo list"
init_logoindex
puts "Initialising channel list"
init_channelindex

foreach file $filelist {
  puts "Processing $file"
  show_import $file
}

init_logoindex

foreach num [lsort -integer [array names channeltablenum]] {
  set sta $channeltablenum($num)
  set data $channeltablestation($sta)
  set sign [lindex $data 2]
  set name [lindex $data 3]
  puts -nonewline "Processing channel $sign :"
  set index [find_logomatch $sign $name]
  if {$index != -1} {
    puts "Associating logos with $sign"
    lappend clist "$sta $index"
  } else {
    puts "No logos found for $sign"
  }
  
}

lsort $clist

foreach channel $clist {
  setLogoIndex [lindex $channel 0] [lindex $channel 1]
}

dbclose $db
puts "Done"

